<?php
/**
 * Extreme Simple PKPass Generator
 * 
 * This script generates a PKPass file without using ANY OpenSSL functions.
 * It's designed to work on hosting environments where OpenSSL file handling
 * doesn't work correctly.
 */

// Enable error reporting
ini_set('display_errors', 1);
ini_set('display_startup_errors', 1);
error_reporting(E_ALL);

// Create logs directory if it doesn't exist
$logDir = __DIR__ . '/../logs';
if (!file_exists($logDir)) {
    mkdir($logDir, 0755, true);
}

// Set up logging
$logFile = $logDir . '/extreme_simple_pass.log';
file_put_contents($logFile, "Extreme Simple PKPass Started: " . date('Y-m-d H:i:s') . "\n", FILE_APPEND);

function log_message($message) {
    global $logFile;
    file_put_contents($logFile, date('H:i:s') . ": $message\n", FILE_APPEND);
}

// Load functions
require_once __DIR__ . '/../includes/functions.php';
log_message("Functions loaded");

// Check for card_number parameter
if (!isset($_GET['card_number'])) {
    log_message("Error: Missing card_number parameter");
    die("Card number is required");
}

$cardNumber = $_GET['card_number'];
log_message("Card number: $cardNumber");

// Get card details from database
try {
    $conn = getDbConnection();
    $stmt = $conn->prepare("
        SELECT lc.*, c.name, c.email, l.name as location_name, l.reward_threshold
        FROM loyalty_cards lc
        JOIN customers c ON lc.customer_id = c.id
        JOIN locations l ON c.location_id = l.id
        WHERE lc.card_number = :card_number
    ");
    $stmt->bindParam(':card_number', $cardNumber);
    $stmt->execute();
    $card = $stmt->fetch(PDO::FETCH_ASSOC);
    
    if (!$card) {
        log_message("Error: Invalid card number");
        die("Invalid card number");
    }
    
    log_message("Card found for: " . $card['name']);
} catch (Exception $e) {
    log_message("Database error: " . $e->getMessage());
    die("Database error: " . $e->getMessage());
}

// Create temporary directory
$tempDir = __DIR__ . '/../temp';
if (!file_exists($tempDir)) {
    mkdir($tempDir, 0755, true);
}

// Create a unique directory for this pass
$passDir = $tempDir . '/extreme_pass_' . uniqid();
mkdir($passDir, 0755, true);
log_message("Created pass directory: $passDir");

// Create pass.json content - VERY minimal
$passJson = '{
  "formatVersion": 1,
  "passTypeIdentifier": "pass.com.example.passbooktest",
  "serialNumber": "' . $cardNumber . '",
  "teamIdentifier": "ABCDEF12345",
  "organizationName": "' . addslashes($card['location_name']) . '",
  "description": "Loyalty Card",
  "logoText": "' . addslashes($card['location_name']) . '",
  "foregroundColor": "rgb(255, 255, 255)",
  "backgroundColor": "rgb(60, 60, 60)",
  "labelColor": "rgb(255, 255, 255)",
  "barcodes": [
    {
      "format": "PKBarcodeFormatPDF417",
      "message": "' . $cardNumber . '",
      "messageEncoding": "iso-8859-1",
      "altText": "' . $cardNumber . '"
    }
  ],
  "barcode": {
    "format": "PKBarcodeFormatPDF417",
    "message": "' . $cardNumber . '",
    "messageEncoding": "iso-8859-1",
    "altText": "' . $cardNumber . '"
  },
  "storeCard": {
    "primaryFields": [
      {
        "key": "member",
        "label": "MEMBER",
        "value": "' . addslashes($card['name']) . '"
      }
    ],
    "secondaryFields": [
      {
        "key": "points",
        "label": "STAMPS",
        "value": "' . $card['stamp_count'] . '/' . $card['reward_threshold'] . '"
      }
    ]
  }
}';

// Write pass.json to file
file_put_contents($passDir . '/pass.json', $passJson);
log_message("Created pass.json");

// Create a 1x1 transparent PNG
$transparentPng = base64_decode('iVBORw0KGgoAAAANSUhEUgAAAAEAAAABCAQAAAC1HAwCAAAAC0lEQVR42mNkYAAAAAYAAjCB0C8AAAAASUVORK5CYII=');

// Save minimal required images
file_put_contents($passDir . '/icon.png', $transparentPng);
file_put_contents($passDir . '/logo.png', $transparentPng);
log_message("Created minimal image files");

// Try to copy existing images if available
$passImagesDir = __DIR__ . '/../assets/pass_images';
$imageFiles = ['icon.png', 'logo.png', 'strip.png', 'icon@2x.png', 'logo@2x.png'];

foreach ($imageFiles as $image) {
    if (file_exists($passImagesDir . '/' . $image)) {
        copy($passImagesDir . '/' . $image, $passDir . '/' . $image);
        log_message("Copied image: $image");
    }
}

// Create manifest.json
$manifest = [];
$files = scandir($passDir);
foreach ($files as $file) {
    if ($file != '.' && $file != '..' && $file != 'manifest.json' && $file != 'signature') {
        $manifest[$file] = sha1_file($passDir . '/' . $file);
    }
}
file_put_contents($passDir . '/manifest.json', json_encode($manifest));
log_message("Created manifest.json");

// Create an empty signature file
file_put_contents($passDir . '/signature', '');
log_message("Created empty signature file");

// Now create the ZIP file (.pkpass)
$pkpassPath = $tempDir . '/pass_' . preg_replace('/[^a-z0-9]/i', '_', $cardNumber) . '.pkpass';

// Remove existing file if it exists
if (file_exists($pkpassPath)) {
    unlink($pkpassPath);
}

$success = false;

// Method 1: Try ZipArchive class
if (class_exists('ZipArchive') && !$success) {
    log_message("Trying ZipArchive method");
    
    try {
        $zip = new ZipArchive();
        if ($zip->open($pkpassPath, ZipArchive::CREATE) === TRUE) {
            $files = scandir($passDir);
            foreach ($files as $file) {
                if ($file != '.' && $file != '..') {
                    $zip->addFile($passDir . '/' . $file, $file);
                }
            }
            $zip->close();
            
            if (file_exists($pkpassPath) && filesize($pkpassPath) > 0) {
                log_message("Created .pkpass file using ZipArchive (" . filesize($pkpassPath) . " bytes)");
                $success = true;
            }
        }
    } catch (Exception $e) {
        log_message("ZipArchive error: " . $e->getMessage());
    }
}

// Method 2: Try command line zip
if (!$success) {
    log_message("Trying command line zip");
    
    $currentDir = getcwd();
    chdir($passDir);
    
    $cmd = "zip -r " . escapeshellarg($pkpassPath) . " .";
    exec($cmd, $output, $returnCode);
    
    chdir($currentDir);
    
    if ($returnCode === 0 && file_exists($pkpassPath) && filesize($pkpassPath) > 0) {
        log_message("Created .pkpass file using command line zip (" . filesize($pkpassPath) . " bytes)");
        $success = true;
    } else {
        log_message("Command line zip failed: " . implode("\n", $output));
    }
}

// Method 3: Try pure PHP ZIP creation (very basic)
if (!$success) {
    log_message("Trying pure PHP zip creation");
    
    $zipData = '';
    $centralDir = '';
    $centralDirSize = 0;
    $centralDirOffset = 0;
    $fileCount = 0;
    
    $files = scandir($passDir);
    foreach ($files as $file) {
        if ($file != '.' && $file != '..') {
            $filePath = $passDir . '/' . $file;
            $content = file_get_contents($filePath);
            
            // Local file header
            $localHeader = "\x50\x4b\x03\x04"; // Local file header signature
            $localHeader .= "\x0a\x00"; // Version needed to extract
            $localHeader .= "\x00\x00"; // General purpose bit flag
            $localHeader .= "\x00\x00"; // Compression method (store)
            $localHeader .= "\x00\x00\x00\x00"; // File last modification time and date
            
            $crc = crc32($content);
            $localHeader .= pack("V", $crc); // CRC-32
            $localHeader .= pack("V", strlen($content)); // Compressed size
            $localHeader .= pack("V", strlen($content)); // Uncompressed size
            $localHeader .= pack("v", strlen($file)); // Filename length
            $localHeader .= pack("v", 0); // Extra field length
            
            // Central directory file header
            $centralDirEntry = "\x50\x4b\x01\x02"; // Central directory file header signature
            $centralDirEntry .= "\x14\x00"; // Version made by
            $centralDirEntry .= "\x0a\x00"; // Version needed to extract
            $centralDirEntry .= "\x00\x00"; // General purpose bit flag
            $centralDirEntry .= "\x00\x00"; // Compression method
            $centralDirEntry .= "\x00\x00\x00\x00"; // File last modification time and date
            $centralDirEntry .= pack("V", $crc); // CRC-32
            $centralDirEntry .= pack("V", strlen($content)); // Compressed size
            $centralDirEntry .= pack("V", strlen($content)); // Uncompressed size
            $centralDirEntry .= pack("v", strlen($file)); // Filename length
            $centralDirEntry .= pack("v", 0); // Extra field length
            $centralDirEntry .= pack("v", 0); // File comment length
            $centralDirEntry .= pack("v", 0); // Disk number start
            $centralDirEntry .= pack("v", 0); // Internal file attributes
            $centralDirEntry .= pack("V", 0); // External file attributes
            $centralDirEntry .= pack("V", $centralDirOffset); // Relative offset of local header
            
            $centralDirOffset += strlen($localHeader) + strlen($file) + strlen($content);
            $zipData .= $localHeader . $file . $content;
            $centralDir .= $centralDirEntry . $file;
            $centralDirSize += strlen($centralDirEntry) + strlen($file);
            $fileCount++;
        }
    }
    
    // End of central directory record
    $endOfCentralDir = "\x50\x4b\x05\x06"; // End of central directory signature
    $endOfCentralDir .= pack("v", 0); // Number of this disk
    $endOfCentralDir .= pack("v", 0); // Disk where central directory starts
    $endOfCentralDir .= pack("v", $fileCount); // Number of central directory records on this disk
    $endOfCentralDir .= pack("v", $fileCount); // Total number of central directory records
    $endOfCentralDir .= pack("V", $centralDirSize); // Size of central directory
    $endOfCentralDir .= pack("V", $centralDirOffset); // Offset of start of central directory
    $endOfCentralDir .= pack("v", 0); // Comment length
    
    $zipData .= $centralDir . $endOfCentralDir;
    
    file_put_contents($pkpassPath, $zipData);
    
    if (file_exists($pkpassPath) && filesize($pkpassPath) > 0) {
        log_message("Created .pkpass file using pure PHP (" . filesize($pkpassPath) . " bytes)");
        $success = true;
    } else {
        log_message("Pure PHP zip creation failed");
    }
}

// Check if we successfully created the .pkpass file
if (!$success) {
    log_message("Failed to create .pkpass file");
    die("Failed to create .pkpass file. Please try again later.");
}

// Clean output buffer before sending file
if (ob_get_level()) {
    ob_end_clean();
}

// Set headers for direct download
header('Content-Description: File Transfer');
header('Content-Type: application/vnd.apple.pkpass');
header('Content-Disposition: attachment; filename="loyalty_card.pkpass"');
header('Content-Length: ' . filesize($pkpassPath));
header('Content-Transfer-Encoding: binary');
header('Connection: Keep-Alive');
header('Expires: 0');
header('Cache-Control: must-revalidate, post-check=0, pre-check=0');
header('Pragma: public');

// Send file to browser
readfile($pkpassPath);

// Log completion
log_message("Pass file sent to browser: " . filesize($pkpassPath) . " bytes");

// Cleanup
foreach (scandir($passDir) as $file) {
    if ($file != '.' && $file != '..') {
        @unlink($passDir . '/' . $file);
    }
}
@rmdir($passDir);

exit; 