<?php
/**
 * PKPass Signature Test with Direct Memory Approach
 * 
 * This file tests the signature creation for PKPass files using a different approach
 * that avoids file path issues on hosting environments with restrictions.
 */

// Enable error reporting
ini_set('display_errors', 1);
ini_set('display_startup_errors', 1);
error_reporting(E_ALL);

// Set up basic HTML structure
?>
<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title>PKPass Signature Fix and Test</title>
    <style>
        body {
            font-family: Arial, sans-serif;
            line-height: 1.6;
            margin: 0;
            padding: 20px;
            color: #333;
            background: #f5f5f5;
        }
        .container {
            max-width: 800px;
            margin: 0 auto;
            background: white;
            padding: 20px;
            border-radius: 5px;
            box-shadow: 0 2px 5px rgba(0,0,0,0.1);
        }
        h1, h2, h3 {
            color: #444;
        }
        .success {
            color: #2e7d32;
            background: #e8f5e9;
            padding: 10px;
            border-radius: 4px;
        }
        .error {
            color: #c62828;
            background: #ffebee;
            padding: 10px;
            border-radius: 4px;
        }
        .warning {
            color: #ef6c00;
            background: #fff3e0;
            padding: 10px;
            border-radius: 4px;
        }
        .section {
            margin: 20px 0;
            padding: 15px;
            border: 1px solid #ddd;
            border-radius: 4px;
        }
        .button {
            display: inline-block;
            background: #1976d2;
            color: white;
            padding: 10px 15px;
            text-decoration: none;
            border-radius: 4px;
            margin: 10px 5px 10px 0;
        }
        pre {
            background: #f5f5f5;
            padding: 10px;
            overflow: auto;
            border-radius: 4px;
        }
    </style>
</head>
<body>
    <div class="container">
        <h1>PKPass Signature Test and Fix</h1>
        <p>This tool tests signature creation for PKPass files with a direct memory approach to avoid file path issues.</p>
        
        <?php
        // Start time
        $startTime = microtime(true);
        echo "<p>Test started at: " . date('Y-m-d H:i:s') . "</p>";
        
        // Check OpenSSL
        echo "<div class='section'>";
        echo "<h2>OpenSSL Check</h2>";
        
        if (extension_loaded('openssl')) {
            echo "<p class='success'>✓ OpenSSL extension is loaded.</p>";
            echo "<p>OpenSSL Version: " . OPENSSL_VERSION_TEXT . "</p>";
        } else {
            echo "<p class='error'>✗ OpenSSL extension is not loaded!</p>";
            echo "</div>";
            echo "</div></body></html>";
            exit;
        }
        
        // Check certificates
        echo "</div><div class='section'>";
        echo "<h2>Certificate Check</h2>";
        
        $p12Path = __DIR__ . '/../certs/Certificates.p12';
        $p12Password = 'T@pple9696'; // Replace with your actual password
        $wwdrPath = __DIR__ . '/../certs/AppleWWDRMPCA1G1.cer';
        
        if (file_exists($p12Path)) {
            echo "<p class='success'>✓ P12 Certificate found: " . $p12Path . " (" . filesize($p12Path) . " bytes)</p>";
        } else {
            echo "<p class='error'>✗ P12 Certificate not found!</p>";
        }
        
        if (file_exists($wwdrPath)) {
            echo "<p class='success'>✓ WWDR Certificate found: " . $wwdrPath . " (" . filesize($wwdrPath) . " bytes)</p>";
        } else {
            echo "<p class='error'>✗ WWDR Certificate not found!</p>";
        }
        
        if (!file_exists($p12Path) || !file_exists($wwdrPath)) {
            echo "</div>";
            echo "</div></body></html>";
            exit;
        }
        
        // Read P12 certificate with direct approach
        echo "</div><div class='section'>";
        echo "<h2>Reading P12 Certificate</h2>";
        
        $p12Data = file_get_contents($p12Path);
        $result = openssl_pkcs12_read($p12Data, $certs, $p12Password);
        
        if ($result) {
            echo "<p class='success'>✓ Successfully read P12 certificate with password.</p>";
            echo "<p>Certificate contains: " . implode(', ', array_keys($certs)) . "</p>";
        } else {
            echo "<p class='error'>✗ Failed to read P12 certificate: " . openssl_error_string() . "</p>";
            echo "<p>Trying alternative approach...</p>";
            
            // Try with environment variable approach (sometimes works better on some hosts)
            putenv("OPENSSL_CONF=/dev/null");
            $result = openssl_pkcs12_read($p12Data, $certs, $p12Password);
            
            if ($result) {
                echo "<p class='success'>✓ Successfully read P12 certificate with environment variable approach!</p>";
                echo "<p>Certificate contains: " . implode(', ', array_keys($certs)) . "</p>";
            } else {
                echo "<p class='error'>✗ Still failed to read P12 certificate: " . openssl_error_string() . "</p>";
                echo "</div>";
                echo "<div class='section'><h2>Solutions</h2>";
                echo "<p>Since certificate reading failed, you should use the development PKPass version which uses a dummy signature.</p>";
                echo "<p><a href='generate_dev_pkpass.php?card_number=LC202503253423' class='button'>Generate Development PKPass</a></p>";
                echo "</div>";
                echo "</div></body></html>";
                exit;
            }
        }
        
        // Read WWDR certificate
        echo "</div><div class='section'>";
        echo "<h2>Reading WWDR Certificate</h2>";
        
        $wwdrData = file_get_contents($wwdrPath);
        
        if ($wwdrData) {
            echo "<p class='success'>✓ Successfully read WWDR certificate data (" . strlen($wwdrData) . " bytes)</p>";
            
            // Convert DER to PEM
            $wwdrPem = chunk_split(base64_encode($wwdrData), 64, "\n");
            $wwdrPem = "-----BEGIN CERTIFICATE-----\n" . $wwdrPem . "-----END CERTIFICATE-----\n";
            
            // Verify PEM format
            $cert = openssl_x509_read($wwdrPem);
            if ($cert) {
                echo "<p class='success'>✓ Successfully verified WWDR certificate in PEM format.</p>";
                
                // Get certificate info
                $certInfo = openssl_x509_parse($cert);
                if ($certInfo && isset($certInfo['subject']['CN'])) {
                    echo "<p>WWDR Certificate Subject: " . $certInfo['subject']['CN'] . "</p>";
                }
            } else {
                echo "<p class='error'>✗ Failed to verify WWDR certificate: " . openssl_error_string() . "</p>";
            }
        } else {
            echo "<p class='error'>✗ Failed to read WWDR certificate data!</p>";
        }
        
        // Create a manifest
        echo "</div><div class='section'>";
        echo "<h2>Creating Test Manifest</h2>";
        
        $manifest = [
            'test.txt' => sha1('test content')
        ];
        
        $manifestJson = json_encode($manifest);
        echo "<p>Created test manifest: " . htmlspecialchars($manifestJson) . "</p>";
        
        // Create a test file in memory
        $tempFileName = tempnam(sys_get_temp_dir(), 'manifest_');
        file_put_contents($tempFileName, $manifestJson);
        
        echo "<p>Created temporary manifest file at: " . $tempFileName . "</p>";
        
        // Try direct signature approach
        echo "</div><div class='section'>";
        echo "<h2>Testing Direct Signature Approach</h2>";
        
        try {
            // Create temporary files in memory
            $tempSignature = tempnam(sys_get_temp_dir(), 'sig_');
            $tempCertFile = tempnam(sys_get_temp_dir(), 'cert_');
            $tempKeyFile = tempnam(sys_get_temp_dir(), 'key_');
            $tempWwdrFile = tempnam(sys_get_temp_dir(), 'wwdr_');
            
            // Write data to these files
            file_put_contents($tempCertFile, $certs['cert']);
            file_put_contents($tempKeyFile, $certs['pkey']);
            file_put_contents($tempWwdrFile, $wwdrPem);
            
            echo "<p>Created temporary certificate files.</p>";
            
            // Try to sign
            $signResult = openssl_pkcs7_sign(
                $tempFileName,
                $tempSignature,
                $certs['cert'],
                [$certs['pkey'], $p12Password],
                [],
                PKCS7_BINARY | PKCS7_DETACHED
            );
            
            if ($signResult && file_exists($tempSignature) && filesize($tempSignature) > 0) {
                echo "<p class='success'>✓ Successfully created signature using in-memory approach!</p>";
                
                // Try to extract DER data
                $signatureContent = file_get_contents($tempSignature);
                if (preg_match('~Content-Disposition:[^\n]+\s*?([A-Za-z0-9+=/\r\n]+)\s*?-----~s', $signatureContent, $matches)) {
                    $derSignature = base64_decode(preg_replace('/\s+/', '', $matches[1]));
                    $derSize = strlen($derSignature);
                    echo "<p class='success'>✓ Successfully extracted DER data from PEM signature (" . $derSize . " bytes)</p>";
                    
                    // Create a sample signature
                    $sampleSignaturePath = __DIR__ . '/../temp/sample_signature';
                    file_put_contents($sampleSignaturePath, $derSignature);
                    echo "<p>Saved sample signature to: " . $sampleSignaturePath . "</p>";
                } else {
                    echo "<p class='error'>✗ Failed to extract DER data from PEM signature</p>";
                }
            } else {
                echo "<p class='error'>✗ Failed to create signature: " . openssl_error_string() . "</p>";
                echo "<p>Creating a dummy signature instead...</p>";
                
                // Create a dummy signature for testing
                $dummySignature = hex2bin(
                    "3082028006092a864886f70d010702a082027130820" .
                    "26d02010131180f32303135303631343139313934" .
                    "315a3082023c308202383082017ea00302010202" .
                    "100b53992c61cae4336bf7d540a3b890a300d06" .
                    "092a864886f70d01010b0500301b311930170603" .
                    "55040313104465762043657274696669636174653" .
                    "0301e170d3235303332353130303030305a170d33" .
                    "30303332353130303030305a301b3119301706035" .
                    "5040313104465762043657274696669636174653" .
                    "0819f300d06092a864886f70d010101050003818d" .
                    "0030818902818100c52e016c01eb7d47ba05b0149" .
                    "49c71f01c94da0172c0bac106e643de1f0daa2ca4" .
                    "1f4a97a6b4ba4e7e14b8da61d29819c0c8ed91a84" .
                    "05b9c21f8c8ddbfa3b8c46412c0b3ed81f712719b" .
                    "ca24482fa7a1b77ad32b333379d2c1bcf5e0e64a3" .
                    "fef5981ed2a3a190aed2edebb676257f2f9271e8a" .
                    "3b1b05a8b0203010001a321301f301d0603551d0e" .
                    "0416041417814c55aa31144f0b01c78deaffeaf47" .
                    "3867b3300d06092a864886f70d01010b05000381" .
                    "810081dc6c0708b83d9a3c534c7ffd42dbd5c3dd" .
                    "c78133281c1ef8ad7adbc266e0ceebd969a1e900" .
                    "ba1759ba8b00993faef04b4e6c14e7de8e8f60ad" .
                    "b5ec1a40dc66a5a33fb584e541f3b8d5a19aa5c0" .
                    "a0c76f51ea989aa9948a4e4ef78e98c16c60d4b6" .
                    "7cc7bd9916b8cb2cbccfeeb2b3c0c0e41e66bec9" .
                    "eeb65ca5afbd6de57"
                );
                
                $dummySignaturePath = __DIR__ . '/../temp/dummy_signature';
                file_put_contents($dummySignaturePath, $dummySignature);
                echo "<p class='success'>✓ Created dummy signature for testing (" . strlen($dummySignature) . " bytes)</p>";
                echo "<p>Dummy signature path: " . $dummySignaturePath . "</p>";
            }
            
            // Clean up temporary files
            @unlink($tempFileName);
            @unlink($tempSignature);
            @unlink($tempCertFile);
            @unlink($tempKeyFile);
            @unlink($tempWwdrFile);
        } catch (Exception $e) {
            echo "<p class='error'>Exception: " . $e->getMessage() . "</p>";
        }
        
        // Conclusion and next steps
        echo "</div><div class='section'>";
        echo "<h2>Conclusion and Next Steps</h2>";
        echo "<p>Based on the tests above, here are your options:</p>";
        
        echo "<ol>";
        echo "<li><strong>Use the Development PKPass class</strong> - This bypasses certificate validation issues with a dummy signature:</li>";
        echo "<p><a href='generate_dev_pkpass.php?card_number=LC202503253423' class='button'>Generate Development PKPass</a></p>";
        
        if (isset($signResult) && $signResult) {
            echo "<li><strong>Try the Production PKPass generator</strong> - Certificate validation might work:</li>";
            echo "<p><a href='generate_proper_pkpass.php?card_number=LC202503253423' class='button' style='background-color: #388e3c;'>Generate Production PKPass</a></p>";
        } else {
            echo "<li><strong>Production PKPass generator</strong> - Not recommended based on this test.</li>";
        }
        echo "</ol>";
        
        $endTime = microtime(true);
        $executionTime = round($endTime - $startTime, 2);
        echo "<p>Test completed in $executionTime seconds.</p>";
        echo "</div>";
        
        ?>
    </div>
</body>
</html>
<?php
// Clean up any remaining temp files
@unlink($tempFileName);
?> 