<?php
/**
 * Generate Development PKPass
 * 
 * This script generates Apple Wallet passes for development purposes without requiring
 * certificate validation. It creates a technically invalid but workable PKPass file
 * for testing on development environments where certificate issues may occur.
 * 
 * NOTE: This should ONLY be used for development. Do not use in production!
 */

// Enable error reporting for debugging
ini_set('display_errors', 1);
ini_set('display_startup_errors', 1);
error_reporting(E_ALL);

// Set up logging
$logDir = __DIR__ . '/../logs';
if (!file_exists($logDir)) {
    mkdir($logDir, 0755, true);
}

$logFile = $logDir . '/dev_pkpass.log';
file_put_contents($logFile, "Starting development PKPass generation: " . date('Y-m-d H:i:s') . "\n", FILE_APPEND);

// Function to log messages
function log_message($message) {
    global $logFile;
    file_put_contents($logFile, $message . "\n", FILE_APPEND);
    return $message;
}

require_once __DIR__ . '/../includes/functions.php';
log_message("Loaded functions.php");

require_once __DIR__ . '/pkpass_dev.php';
log_message("Loaded PKPass_Dev class");

// Check if card_number parameter is provided
if (!isset($_GET['card_number'])) {
    log_message("Error: Card number parameter is missing");
    die("Card number parameter is required");
}

$cardNumber = $_GET['card_number'];
log_message("Card Number: $cardNumber");

// Get card details
try {
    $conn = getDbConnection();
    log_message("Database connection established");
    
    $stmt = $conn->prepare("
        SELECT lc.*, c.name, c.email, l.name as location_name, l.logo_url, l.reward_threshold, l.reward_description
        FROM loyalty_cards lc
        JOIN customers c ON lc.customer_id = c.id
        JOIN locations l ON c.location_id = l.id
        WHERE lc.card_number = :card_number
    ");
    $stmt->bindParam(':card_number', $cardNumber);
    $stmt->execute();
    $card = $stmt->fetch(PDO::FETCH_ASSOC);
    
    if (!$card) {
        log_message("Error: Invalid card number - no results found");
        die("Invalid card number");
    }
    
    log_message("Card found: " . json_encode($card));
} catch (Exception $e) {
    log_message("Database error: " . $e->getMessage());
    die("Database error: " . $e->getMessage());
}

// Define the path to pass images
$passImagesDir = __DIR__ . '/../assets/pass_images';

// Create the pass data
$passData = [
    'formatVersion' => 1,
    'passTypeIdentifier' => 'pass.com.example.passbooktest',
    'serialNumber' => $cardNumber . '_DEV_' . time(),
    'teamIdentifier' => 'C7CH6P3GLR',
    'organizationName' => $card['location_name'] . ' (DEV)',
    'description' => 'Loyalty Card for ' . $card['location_name'] . ' (Development Version)',
    'logoText' => $card['location_name'] . ' (DEV)',
    'foregroundColor' => 'rgb(255, 255, 255)',
    'backgroundColor' => 'rgb(67, 97, 238)',
    'labelColor' => 'rgb(255, 255, 255)',
    'webServiceURL' => 'https://dev.trimscloud.com/services/passes/',
    'authenticationToken' => md5($cardNumber . time()),
    'barcodes' => [
        [
            'format' => 'PKBarcodeFormatPDF417',
            'message' => $cardNumber,
            'messageEncoding' => 'iso-8859-1',
            'altText' => $cardNumber . ' (DEV)'
        ]
    ],
    // Include barcode for older iOS versions
    'barcode' => [
        'format' => 'PKBarcodeFormatPDF417',
        'message' => $cardNumber,
        'messageEncoding' => 'iso-8859-1',
        'altText' => $cardNumber . ' (DEV)'
    ],
    'storeCard' => [
        'headerFields' => [
            [
                'key' => 'balance',
                'label' => 'COLLECTED',
                'value' => $card['stamp_count'] . '/' . $card['reward_threshold']
            ]
        ],
        'primaryFields' => [
            [
                'key' => 'offer',
                'label' => 'REWARD',
                'value' => 'Buy ' . ($card['reward_threshold'] - 1) . ' Get 1 FREE!'
            ]
        ],
        'secondaryFields' => [
            [
                'key' => 'name',
                'label' => 'CARDHOLDER',
                'value' => $card['name']
            ]
        ],
        'auxiliaryFields' => [
            [
                'key' => 'stamps',
                'label' => 'STAMPS',
                'value' => $card['stamp_count']
            ]
        ],
        'backFields' => [
            [
                'key' => 'terms',
                'label' => 'TERMS AND CONDITIONS',
                'value' => 'Present this card when making a purchase to collect stamps. Collect ' . $card['reward_threshold'] . ' stamps to get a free item.'
            ],
            [
                'key' => 'website',
                'label' => 'WEBSITE',
                'value' => 'https://dev.trimscloud.com'
            ],
            [
                'key' => 'cardNumber',
                'label' => 'CARD NUMBER',
                'value' => $cardNumber
            ],
            [
                'key' => 'devNote',
                'label' => 'DEVELOPMENT MODE',
                'value' => 'This pass was created in development mode and is for testing purposes only. Created: ' . date('Y-m-d H:i:s')
            ],
            [
                'key' => 'devMode',
                'label' => '⚠️ ATTENTION',
                'value' => 'This pass uses a dummy signature and is not validated by Apple. It may not function correctly in all environments.'
            ]
        ]
    ]
];

log_message("Pass data prepared");

try {
    // Create a new PKPass_Dev instance
    $pass = new PKPass_Dev();
    log_message("PKPass_Dev instance created");
    
    // Set the pass data
    $pass->setData($passData);
    log_message("Pass data set");
    
    // Set the pass images directory
    $pass->setPassImagesPath($passImagesDir);
    log_message("Pass images path set: $passImagesDir");
    
    // Create the pass
    log_message("Creating PKPass file using development mode (with dummy signature)...");
    log_message("IMPORTANT: This pass uses a dummy signature and is for TESTING ONLY!");
    $pkpassPath = $pass->create();
    
    if ($pkpassPath) {
        log_message("Pass file created at: $pkpassPath");
        log_message("Pass file size: " . filesize($pkpassPath) . " bytes");
        
        // Verify the file exists and is readable
        if (file_exists($pkpassPath) && is_readable($pkpassPath) && filesize($pkpassPath) > 0) {
            log_message("Pass file verified. Sending to browser...");
            
            // Add a debug message
            echo "<!-- DEBUG: Pass created, about to send to browser. If you see this message, the pass was not sent correctly. -->";
            
            // Send the file to the browser
            $pass->send($pkpassPath, $card['location_name'] . '_Card.pkpass');
            // The send() method calls exit(), so the script ends here
        } else {
            $error = "Error: Pass file exists but may not be readable or is empty";
            log_message($error);
            die($error);
        }
    } else {
        $error = "Error creating pass: " . $pass->getError();
        log_message($error);
        die($error);
    }
} catch (Exception $e) {
    $error = "Exception: " . $e->getMessage();
    log_message($error);
    log_message("Stack trace: " . $e->getTraceAsString());
    die("Error: " . $e->getMessage());
}
?> 