<?php
// Enable error reporting for debugging
ini_set('display_errors', 1);
ini_set('display_startup_errors', 1);
error_reporting(E_ALL);

$basePath = realpath(dirname(__FILE__) . '/../..');
require_once($basePath . '/includes/functions.php');

// Set up logging
$logFile = $basePath . '/logs/passkit_push.log';
$logDir = dirname($logFile);
if (!is_dir($logDir)) {
    mkdir($logDir, 0777, true);
}

// Log function for debugging
function logMessage($message) {
    global $logFile;
    file_put_contents($logFile, date('Y-m-d H:i:s') . ' - ' . $message . "\n", FILE_APPEND);
}

// Log all incoming requests for debugging
logMessage('REQUEST: ' . $_SERVER['REQUEST_METHOD'] . ' ' . $_SERVER['REQUEST_URI']);
logMessage('HEADERS: ' . json_encode(getallheaders()));

// Connect to database
$conn = getDbConnection();

/**
 * Push updates to all devices registered for this pass
 * 
 * @param string $cardNumber The loyalty card number
 * @return array Response with success status and message
 */
function pushPassUpdates($cardNumber) {
    global $conn;
    logMessage("Pushing updates for card number: $cardNumber");
    
    // Get all registered devices for this pass
    try {
        $stmt = $conn->prepare("
            SELECT device_id, push_token 
            FROM pass_registrations 
            WHERE pass_id = ?
        ");
        $stmt->execute([$cardNumber]);
        $devices = $stmt->fetchAll(PDO::FETCH_ASSOC);
        
        if (empty($devices)) {
            logMessage("No registered devices found for card: $cardNumber");
            return ['success' => false, 'message' => 'No registered devices found'];
        }
        
        logMessage("Found " . count($devices) . " registered device(s)");
        
        // Set certificate paths
        $certDir = $GLOBALS['basePath'] . '/certs/';
        $pushCertPath = $certDir . 'pushCert.pem';
        $pushKeyPath = $certDir . 'pushKey.pem';
        $passphrase = 'test'; // Replace with actual passphrase
        
        if (!file_exists($pushCertPath)) {
            logMessage("Push certificate not found: $pushCertPath");
            return ['success' => false, 'message' => 'Push certificate not found'];
        }
        
        if (!file_exists($pushKeyPath)) {
            logMessage("Push key not found: $pushKeyPath");
            return ['success' => false, 'message' => 'Push key not found'];
        }
        
        $pushedToCount = 0;
        $failedCount = 0;
        
        // Send push notification to each device
        foreach ($devices as $device) {
            $deviceId = $device['device_id'];
            $pushToken = $device['push_token'];
            
            logMessage("Sending push to device: $deviceId with token: $pushToken");
            
            // Production URL
            $url = "https://api.push.apple.com/3/device/$pushToken";
            
            // For development/testing use:
            // $url = "https://api.sandbox.push.apple.com/3/device/$pushToken";
            
            $ch = curl_init($url);
            curl_setopt($ch, CURLOPT_SSLCERT, $pushCertPath);
            curl_setopt($ch, CURLOPT_SSLKEY, $pushKeyPath);
            curl_setopt($ch, CURLOPT_SSLKEYPASSWD, $passphrase);
            curl_setopt($ch, CURLOPT_CUSTOMREQUEST, 'POST');
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
            curl_setopt($ch, CURLOPT_HTTPHEADER, [
                'apns-topic: pass.pointify.pk', // Replace with your pass type identifier
                'apns-priority: 10',
            ]);
            
            // Empty payload for passkit push (the device will request the updated pass)
            curl_setopt($ch, CURLOPT_POSTFIELDS, '{}');
            
            $response = curl_exec($ch);
            $status = curl_getinfo($ch, CURLINFO_HTTP_CODE);
            $error = curl_error($ch);
            
            logMessage("Push response: status=$status, response=$response, error=$error");
            
            if ($status == 200) {
                $pushedToCount++;
                // Update the last_pushed timestamp
                $stmt = $conn->prepare("UPDATE pass_updates SET last_pushed = NOW() WHERE pass_id = ?");
                $stmt->execute([$cardNumber]);
            } else {
                $failedCount++;
                // Handle specific error cases
                if ($status == 410) { // Gone - token is no longer valid
                    logMessage("Device token is no longer valid, removing registration: $deviceId");
                    $stmt = $conn->prepare("DELETE FROM pass_registrations WHERE device_id = ? AND pass_id = ?");
                    $stmt->execute([$deviceId, $cardNumber]);
                }
            }
            
            curl_close($ch);
        }
        
        logMessage("Push completed for $cardNumber: $pushedToCount succeeded, $failedCount failed");
        
        return [
            'success' => $pushedToCount > 0,
            'message' => "Push notification sent to $pushedToCount device(s), failed for $failedCount device(s)"
        ];
    } catch (Exception $e) {
        logMessage("Error pushing updates: " . $e->getMessage());
        return ['success' => false, 'message' => 'Error: ' . $e->getMessage()];
    }
}

// Handle API requests
if ($_SERVER['REQUEST_METHOD'] === 'POST') {
    $requestData = json_decode(file_get_contents('php://input'), true);
    $cardNumber = isset($requestData['card_number']) ? $requestData['card_number'] : '';
    
    if (empty($cardNumber)) {
        logMessage("Missing card_number in request");
        http_response_code(400);
        echo json_encode(['status' => 'error', 'message' => 'Card number is required']);
        exit;
    }
    
    logMessage("Processing push request for card: $cardNumber");
    $result = pushPassUpdates($cardNumber);
    
    if ($result['success']) {
        echo json_encode(['status' => 'success', 'message' => $result['message']]);
    } else {
        http_response_code(500);
        echo json_encode(['status' => 'error', 'message' => $result['message']]);
    }
} else {
    logMessage("Method not allowed: " . $_SERVER['REQUEST_METHOD']);
    http_response_code(405);
    echo json_encode(['status' => 'error', 'message' => 'Method not allowed']);
}
?> 