<?php
// Enable error reporting for debugging
ini_set('display_errors', 1);
ini_set('display_startup_errors', 1);
error_reporting(E_ALL);

// Check if card number is provided
if (!isset($_GET['card_number']) || empty($_GET['card_number'])) {
    die('Card number is required');
}

$cardNumber = $_GET['card_number'];

// Set paths
$certDir = __DIR__ . '/../certs/';
$passDir = __DIR__ . '/../passes/';
$passTemplateDir = $passDir . 'templates/loyalty/';

// Create temporary directory for this pass
$tempPassDir = $passDir . 'tmp/' . uniqid('pass_');
if (!mkdir($tempPassDir, 0777, true)) {
    die("Failed to create temp directory: " . error_get_last()['message']);
}

echo "Temp directory created at: $tempPassDir<br>";

// Copy pass template to temp directory
if (!copy($passTemplateDir . 'pass.json', $tempPassDir . '/pass.json')) {
    die("Failed to copy pass.json: " . error_get_last()['message']);
}

echo "Pass.json copied successfully<br>";

// Copy sample images 
$imageFiles = ['icon.png', 'icon@2x.png', 'logo.png', 'logo@2x.png'];
foreach ($imageFiles as $image) {
    if (file_exists($passTemplateDir . $image)) {
        if (!copy($passTemplateDir . $image, $tempPassDir . '/' . $image)) {
            die("Failed to copy $image: " . error_get_last()['message']);
        }
        echo "Copied $image successfully<br>";
    } else {
        echo "Warning: $image not found in template directory<br>";
    }
}

// Read and update pass.json with actual values
$passJsonPath = $tempPassDir . '/pass.json';
$passJson = json_decode(file_get_contents($passJsonPath), true);
if (!$passJson) {
    die("Error parsing pass.json: " . json_last_error_msg());
}

// Update pass data
$passJson['serialNumber'] = $cardNumber;
$passJson['barcode']['message'] = $cardNumber;
$passJson['barcode']['altText'] = $cardNumber;
$passJson['storeCard']['secondaryFields'][0]['value'] = $cardNumber;

// Write updated pass.json
if (!file_put_contents($passJsonPath, json_encode($passJson))) {
    die("Failed to write updated pass.json");
}
echo "Updated pass.json with card data<br>";

// Create a simple manifest
$manifest = [];
$fileList = glob($tempPassDir . '/*');
foreach ($fileList as $file) {
    if (is_file($file) && basename($file) != 'manifest.json' && basename($file) != 'signature') {
        $manifest[basename($file)] = sha1_file($file);
        echo "Added " . basename($file) . " to manifest<br>";
    }
}

if (!file_put_contents($tempPassDir . '/manifest.json', json_encode($manifest))) {
    die("Failed to write manifest.json: " . error_get_last()['message']);
}

echo "Manifest created successfully<br>";

// Define certificate paths
$signerCertPath = $certDir . 'signerCert.pem';
$signerKeyPath = $certDir . 'signerKey.pem';
$wwdrCertPath = $certDir . 'Apple Worldwide Developer Relations Certification Authority.pem';
$keyPassphrase = 'test'; // The passphrase from your test script

echo "Using certificates:<br>";
echo "Signer Cert: $signerCertPath<br>";
echo "Signer Key: $signerKeyPath<br>";
echo "WWDR Cert: $wwdrCertPath<br>";

// Test if we can read the private key
$pkeyData = file_get_contents($signerKeyPath);
$privateKey = openssl_pkey_get_private($pkeyData, $keyPassphrase);
if (!$privateKey) {
    die("Error loading private key: " . openssl_error_string());
}

echo "Private key loaded successfully<br>";

// Sign the manifest.json file
$signatureFile = $tempPassDir . '/signature';
$manifestData = file_get_contents($tempPassDir . '/manifest.json');
$certData = file_get_contents($signerCertPath);

// Create a temporary file for signing
$tempManifestFile = $passDir . 'tmp/manifest_' . uniqid() . '.json';
if (file_put_contents($tempManifestFile, $manifestData) === false) {
    die("Failed to write to temporary manifest file");
}

echo "Temp manifest file created at: $tempManifestFile<br>";

// Try signing
if (!openssl_pkcs7_sign(
    $tempManifestFile,
    $signatureFile,
    $certData,
    array($privateKey, $keyPassphrase),
    array(),
    PKCS7_BINARY | PKCS7_DETACHED,
    $wwdrCertPath
)) {
    die("Error signing manifest: " . openssl_error_string());
}

echo "Manifest signed successfully<br>";

// Read signature file and strip out the MIME headers
$signature = file_get_contents($signatureFile);
echo "Raw signature file length: " . strlen($signature) . " bytes<br>";

// Extract the signature data
$pattern = '/(Content-Disposition: attachment.*?\n|.*?)\n(.*)/s';
if (preg_match($pattern, $signature, $matches)) {
    $signature = $matches[2];
    echo "Extracted signature data, length: " . strlen($signature) . " bytes<br>";
} else {
    echo "Warning: Could not extract signature with regex pattern<br>";
    // Fallback to simple split
    $parts = explode("\n\n", $signature, 2);
    if (count($parts) > 1) {
        $signature = $parts[1];
        echo "Extracted signature data with fallback method, length: " . strlen($signature) . " bytes<br>";
    } else {
        die("Could not extract signature data from file");
    }
}

// Base64 decode
$decodedSignature = base64_decode($signature);
if ($decodedSignature === false) {
    die("Failed to base64 decode signature");
}

echo "Signature decoded, length: " . strlen($decodedSignature) . " bytes<br>";
file_put_contents($signatureFile, $decodedSignature);

// Create the final .pkpass file
$passFilename = $cardNumber . '.pkpass';
$passFile = $passDir . $passFilename;

echo "Creating .pkpass file at: $passFile<br>";

// Create .pkpass (zip archive)
$zip = new ZipArchive();
if ($zip->open($passFile, ZipArchive::CREATE | ZipArchive::OVERWRITE) !== true) {
    die("Cannot create ZIP archive");
}

// Add all files from the temporary directory
$dirContents = glob($tempPassDir . '/*');
foreach ($dirContents as $file) {
    if (is_file($file)) {
        $zip->addFile($file, basename($file));
        echo "Added " . basename($file) . " to ZIP<br>";
    }
}
$zip->close();

echo "ZIP file created successfully<br>";

// Clean up temporary files
unlink($tempManifestFile);
foreach (glob($tempPassDir . '/*') as $file) {
    unlink($file);
}
rmdir($tempPassDir);

echo "Temp files cleaned up<br>";

// Output success and provide download link
echo "<h2>Pass created successfully!</h2>";
echo "<p>Your pass file is ready. Click the button below to download it:</p>";
echo "<a href='download_pass.php?file=$passFilename' style='display:inline-block; background:#000; color:#fff; padding:10px 20px; text-decoration:none; border-radius:5px;'>Download Pass</a>";
?> 