<?php
// Start output buffering at the very beginning
ob_start();

require_once __DIR__ . '/db_connect.php';

// Start session
if (session_status() == PHP_SESSION_NONE) {
    session_start();
}

// Generate a random OTP
function generateOTP($length = 6) {
    $characters = '0123456789';
    $otp = '';
    
    for ($i = 0; $i < $length; $i++) {
        $otp .= $characters[rand(0, strlen($characters) - 1)];
    }
    
    return $otp;
}

// Store OTP in the database
function storeOTP($customerId, $otp, $expiryMinutes = 15) {
    $conn = getDbConnection();
    
    // Delete any existing OTPs for this customer
    $stmt = $conn->prepare("DELETE FROM otp_verifications WHERE customer_id = :customer_id");
    $stmt->bindParam(':customer_id', $customerId);
    $stmt->execute();
    
    // Calculate expiry time
    $expiryTime = date('Y-m-d H:i:s', strtotime("+{$expiryMinutes} minutes"));
    
    // Insert new OTP
    $stmt = $conn->prepare("
        INSERT INTO otp_verifications (customer_id, otp, expires_at) 
        VALUES (:customer_id, :otp, :expires_at)
    ");
    $stmt->bindParam(':customer_id', $customerId);
    $stmt->bindParam(':otp', $otp);
    $stmt->bindParam(':expires_at', $expiryTime);
    
    return $stmt->execute();
}

// Verify OTP
function verifyOTP($customerId, $otp) {
    $conn = getDbConnection();
    
    // Get the latest OTP for this customer
    $stmt = $conn->prepare("
        SELECT * FROM otp_verifications 
        WHERE customer_id = :customer_id 
        ORDER BY created_at DESC 
        LIMIT 1
    ");
    $stmt->bindParam(':customer_id', $customerId);
    $stmt->execute();
    
    $verification = $stmt->fetch(PDO::FETCH_ASSOC);
    
    if (!$verification) {
        return false;
    }
    
    // Check if OTP matches and is not expired
    $now = new DateTime();
    $expiryTime = new DateTime($verification['expires_at']);
    
    // Trim both values and use strict comparison
    $storedOtp = trim($verification['otp']);
    $inputOtp = trim($otp);
    
    if ($storedOtp === $inputOtp && $now <= $expiryTime) {
        // Delete the OTP after successful verification
        $stmt = $conn->prepare("DELETE FROM otp_verifications WHERE id = :id");
        $stmt->bindParam(':id', $verification['id']);
        $stmt->execute();
        
        return true;
    }
    
    return false;
}

// Generate a unique card number
function generateCardNumber() {
    return 'LC' . date('Ymd') . rand(1000, 9999);
}

// Get location by slug
function getLocationBySlug($slug) {
    $conn = getDbConnection();
    
    $stmt = $conn->prepare("SELECT * FROM locations WHERE slug = :slug");
    $stmt->bindParam(':slug', $slug);
    $stmt->execute();
    
    return $stmt->fetch(PDO::FETCH_ASSOC);
}

// Send email with OTP
function sendOTPEmail($email, $name, $otp) {
    global $config;
    
    $subject = "Your Verification Code";
    $message = "
    <html>
    <head>
        <title>Email Verification</title>
    </head>
    <body>
        <p>Hello {$name},</p>
        <p>Your verification code is: <strong>{$otp}</strong></p>
        <p>This code will expire in {$config['otp']['expiry']} minutes.</p>
        <p>Thank you for registering with our Pointify!</p>
    </body>
    </html>
    ";
    
    // For development, log the email instead of sending it
    if ($_SERVER['SERVER_NAME'] == 'localhost') {
        error_log("Email to: {$email}, Subject: {$subject}, OTP: {$otp}");
        return true;
    }
    
    // Headers
    $headers = "MIME-Version: 1.0" . "\r\n";
    $headers .= "Content-type:text/html;charset=UTF-8" . "\r\n";
    $headers .= "From: {$config['email']['name']} <{$config['email']['from']}>" . "\r\n";
    
    // SMTP settings for mail()
    ini_set("SMTP", $config['email']['smtp']['host']);
    ini_set("smtp_port", $config['email']['smtp']['port']);
    ini_set("sendmail_from", $config['email']['from']);
    
    // Basic authentication
    $auth = base64_encode("{$config['email']['smtp']['username']}:{$config['email']['smtp']['password']}");
    $headers .= "Authorization: Basic " . $auth . "\r\n";
    
    // Send email
    $success = mail($email, $subject, $message, $headers);
    
    if (!$success) {
        error_log("Failed to send email to {$email}. Error: " . error_get_last()['message']);
        // For development, still log the OTP
        error_log("OTP for testing: {$otp}");
    }
    
    return $success;
}

// Generate PDF417 barcode data
function generatePDF417BarcodeData($cardNumber) {
    // In a real application, you would use a library like picqer/php-barcode-generator
    // For this example, we'll return a placeholder URL to a barcode generator service
    return "https://barcodeapi.org/api/pdf417/{$cardNumber}";
}

// Create a new customer
function createCustomer($name, $email, $locationId) {
    $conn = getDbConnection();
    
    try {
        $conn->beginTransaction();
        
        // Check if email already exists
        $stmt = $conn->prepare("SELECT id FROM customers WHERE email = :email");
        $stmt->bindParam(':email', $email);
        $stmt->execute();
        
        if ($stmt->rowCount() > 0) {
            return ['success' => false, 'message' => 'Email already registered'];
        }
        
        // Insert customer
        $stmt = $conn->prepare("
            INSERT INTO customers (name, email, location_id) 
            VALUES (:name, :email, :location_id)
        ");
        $stmt->bindParam(':name', $name);
        $stmt->bindParam(':email', $email);
        $stmt->bindParam(':location_id', $locationId);
        $stmt->execute();
        
        $customerId = $conn->lastInsertId();
        
        $conn->commit();
        
        return ['success' => true, 'customer_id' => $customerId];
    } catch (PDOException $e) {
        $conn->rollBack();
        return ['success' => false, 'message' => $e->getMessage()];
    }
}

// Create loyalty card for customer
function createLoyaltyCard($customerId) {
    $conn = getDbConnection();
    
    try {
        // Generate unique card number
        $cardNumber = generateCardNumber();
        
        // Insert loyalty card
        $stmt = $conn->prepare("
            INSERT INTO loyalty_cards (customer_id, card_number) 
            VALUES (:customer_id, :card_number)
        ");
        $stmt->bindParam(':customer_id', $customerId);
        $stmt->bindParam(':card_number', $cardNumber);
        $stmt->execute();
        
        return ['success' => true, 'card_number' => $cardNumber];
    } catch (PDOException $e) {
        return ['success' => false, 'message' => $e->getMessage()];
    }
}

// Mark customer as verified
function verifyCustomer($customerId) {
    $conn = getDbConnection();
    
    $stmt = $conn->prepare("
        UPDATE customers 
        SET is_verified = TRUE 
        WHERE id = :customer_id
    ");
    $stmt->bindParam(':customer_id', $customerId);
    
    return $stmt->execute();
}

// Get customer by ID
function getCustomerById($customerId) {
    $conn = getDbConnection();
    
    $stmt = $conn->prepare("SELECT * FROM customers WHERE id = :id");
    $stmt->bindParam(':id', $customerId);
    $stmt->execute();
    
    return $stmt->fetch(PDO::FETCH_ASSOC);
}

// Get loyalty card by customer ID
function getLoyaltyCardByCustomerId($customerId) {
    $conn = getDbConnection();
    
    $stmt = $conn->prepare("SELECT * FROM loyalty_cards WHERE customer_id = :customer_id");
    $stmt->bindParam(':customer_id', $customerId);
    $stmt->execute();
    
    return $stmt->fetch(PDO::FETCH_ASSOC);
}

// Add a stamp to a loyalty card
function addStamp($cardId) {
    $conn = getDbConnection();
    
    try {
        // Start transaction
        $conn->beginTransaction();
        
        // Get current card info and reward threshold
        $stmt = $conn->prepare("
            SELECT lc.*, l.reward_threshold 
            FROM loyalty_cards lc
            JOIN customers c ON lc.customer_id = c.id
            JOIN locations l ON c.location_id = l.id
            WHERE lc.id = :card_id
        ");
        $stmt->bindParam(':card_id', $cardId);
        $stmt->execute();
        $card = $stmt->fetch(PDO::FETCH_ASSOC);
        
        if (!$card) {
            $conn->rollBack();
            return false;
        }
        
        $newStampCount = $card['stamp_count'] + 1;
        $rewardEarned = false;
        
        // Check if reward threshold is reached
        if ($newStampCount >= $card['reward_threshold']) {
            // Reset stamp count to 0 and increment rewards earned
            $newStampCount = 0;
            $rewardEarned = true;
            
            $stmt = $conn->prepare("
                UPDATE loyalty_cards 
                SET stamp_count = :stamp_count, 
                    rewards_earned = rewards_earned + 1 
                WHERE id = :card_id
            ");
        } else {
            // Just update stamp count
            $stmt = $conn->prepare("
                UPDATE loyalty_cards 
                SET stamp_count = :stamp_count 
                WHERE id = :card_id
            ");
        }
        
        $stmt->bindParam(':stamp_count', $newStampCount);
        $stmt->bindParam(':card_id', $cardId);
        $stmt->execute();
        
        // Update the pass_updates table to indicate the pass needs an update
        try {
            $stmt = $conn->prepare("
                INSERT INTO pass_updates (pass_id, last_updated, version) 
                VALUES (:pass_id, NOW(), 1)
                ON DUPLICATE KEY UPDATE 
                last_updated = NOW(), 
                version = version + 1
            ");
            $stmt->bindParam(':pass_id', $card['card_number']);
            $stmt->execute();
            
            // Send push notification for the updated pass
            triggerPassNotification($card['card_number']);
        } catch (Exception $e) {
            // Log the error, but don't fail the stamp update
            error_log("Error updating pass data for push notification: " . $e->getMessage());
        }
        
        $conn->commit();
        return true;
    } catch (PDOException $e) {
        $conn->rollBack();
        error_log("Error adding stamp: " . $e->getMessage());
        return false;
    }
}

// Remove stamp from loyalty card
function removeStamp($cardId) {
    $conn = getDbConnection();
    
    // First check if there are stamps to remove
    $stmt = $conn->prepare("SELECT stamp_count, card_number FROM loyalty_cards WHERE id = :card_id");
    $stmt->bindParam(':card_id', $cardId);
    $stmt->execute();
    $card = $stmt->fetch(PDO::FETCH_ASSOC);
    
    if ($card && $card['stamp_count'] > 0) {
        $stmt = $conn->prepare("UPDATE loyalty_cards SET stamp_count = stamp_count - 1 WHERE id = :card_id");
        $stmt->bindParam(':card_id', $cardId);
        $result = $stmt->execute();
        
        if ($result) {
            // Update the pass_updates table and trigger notification
            try {
                $stmt = $conn->prepare("
                    INSERT INTO pass_updates (pass_id, last_updated, version) 
                    VALUES (:pass_id, NOW(), 1)
                    ON DUPLICATE KEY UPDATE 
                    last_updated = NOW(), 
                    version = version + 1
                ");
                $stmt->bindParam(':pass_id', $card['card_number']);
                $stmt->execute();
                
                // Send push notification for the updated pass
                triggerPassNotification($card['card_number']);
            } catch (Exception $e) {
                // Log the error, but don't fail the stamp removal
                error_log("Error updating pass data for push notification: " . $e->getMessage());
            }
        }
        
        return $result;
    }
    
    return false;
}

/**
 * Trigger a push notification to update a pass
 *
 * @param string $cardNumber The loyalty card number
 * @return bool Success indicator
 */
function triggerPassNotification($cardNumber) {
    // Make a POST request to our push notification API
    $apiUrl = 'https://dev.trimscloud.com/webservice/v1/passes/push.php';
    $data = json_encode(['pass_id' => $cardNumber]);
    
    // Use cURL to make the request
    $ch = curl_init($apiUrl);
    curl_setopt($ch, CURLOPT_CUSTOMREQUEST, 'POST');
    curl_setopt($ch, CURLOPT_POSTFIELDS, $data);
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
    curl_setopt($ch, CURLOPT_HTTPHEADER, [
        'Content-Type: application/json',
        'Content-Length: ' . strlen($data)
    ]);
    
    // Execute and get results
    $response = curl_exec($ch);
    $status = curl_getinfo($ch, CURLINFO_HTTP_CODE);
    curl_close($ch);
    
    if ($status >= 200 && $status < 300) {
        error_log("Successfully triggered pass update for card: $cardNumber");
        return true;
    } else {
        error_log("Failed to trigger pass update for card: $cardNumber. Response: $response");
        return false;
    }
}

// Authenticate user
function authenticateUser($username, $password) {
    $conn = getDbConnection();
    
    $stmt = $conn->prepare("SELECT * FROM users WHERE username = :username");
    $stmt->bindParam(':username', $username);
    $stmt->execute();
    
    $user = $stmt->fetch(PDO::FETCH_ASSOC);
    
    if ($user && password_verify($password, $user['password'])) {
        return $user;
    }
    
    return false;
}

// Check if user is logged in
function checkUserSession() {
    return isset($_SESSION['user_id']);
}

// Check if user is super admin
function isSuperAdmin() {
    if (!checkUserSession()) {
        return false;
    }
    
    return $_SESSION['user_role'] === 'super_admin';
}

// Check if user is location admin
function isLocationAdmin() {
    if (!checkUserSession()) {
        return false;
    }
    
    return $_SESSION['user_role'] === 'location_admin';
}

// Get all locations
function getAllLocations() {
    $conn = getDbConnection();
    
    $stmt = $conn->prepare("SELECT * FROM locations ORDER BY name");
    $stmt->execute();
    
    return $stmt->fetchAll(PDO::FETCH_ASSOC);
}

// Get location by ID
function getLocationById($locationId) {
    $conn = getDbConnection();
    
    $stmt = $conn->prepare("SELECT * FROM locations WHERE id = :id");
    $stmt->bindParam(':id', $locationId);
    $stmt->execute();
    
    return $stmt->fetch(PDO::FETCH_ASSOC);
}

// Create new location
function createLocation($name, $slug, $rewardThreshold = 7) {
    $conn = getDbConnection();
    
    try {
        // Check if slug already exists
        $stmt = $conn->prepare("SELECT id FROM locations WHERE slug = :slug");
        $stmt->bindParam(':slug', $slug);
        $stmt->execute();
        
        if ($stmt->rowCount() > 0) {
            return ['success' => false, 'message' => 'Location slug already exists'];
        }
        
        $registrationLink = BASE_URL . 'customer/register.php?location=' . $slug;
        
        // Insert location
        $stmt = $conn->prepare("
            INSERT INTO locations (name, slug, registration_link, reward_threshold) 
            VALUES (:name, :slug, :registration_link, :reward_threshold)
        ");
        $stmt->bindParam(':name', $name);
        $stmt->bindParam(':slug', $slug);
        $stmt->bindParam(':registration_link', $registrationLink);
        $stmt->bindParam(':reward_threshold', $rewardThreshold);
        $stmt->execute();
        
        return ['success' => true, 'location_id' => $conn->lastInsertId()];
    } catch (PDOException $e) {
        return ['success' => false, 'message' => $e->getMessage()];
    }
}

// Create location admin
function createLocationAdmin($username, $password, $name, $email, $locationId) {
    $conn = getDbConnection();
    
    try {
        // Check if username or email already exists
        $stmt = $conn->prepare("
            SELECT id FROM users 
            WHERE username = :username OR email = :email
        ");
        $stmt->bindParam(':username', $username);
        $stmt->bindParam(':email', $email);
        $stmt->execute();
        
        if ($stmt->rowCount() > 0) {
            return ['success' => false, 'message' => 'Username or email already exists'];
        }
        
        $hashedPassword = password_hash($password, PASSWORD_DEFAULT);
        
        // Insert user
        $stmt = $conn->prepare("
            INSERT INTO users (username, password, name, email, role, location_id) 
            VALUES (:username, :password, :name, :email, 'location_admin', :location_id)
        ");
        $stmt->bindParam(':username', $username);
        $stmt->bindParam(':password', $hashedPassword);
        $stmt->bindParam(':name', $name);
        $stmt->bindParam(':email', $email);
        $stmt->bindParam(':location_id', $locationId);
        $stmt->execute();
        
        return ['success' => true, 'user_id' => $conn->lastInsertId()];
    } catch (PDOException $e) {
        return ['success' => false, 'message' => $e->getMessage()];
    }
}

// Get customers by location
function getCustomersByLocation($locationId) {
    $conn = getDbConnection();
    
    $stmt = $conn->prepare("
        SELECT c.*, lc.card_number, lc.stamp_count 
        FROM customers c
        LEFT JOIN loyalty_cards lc ON c.id = lc.customer_id
        WHERE c.location_id = :location_id
        ORDER BY c.name
    ");
    $stmt->bindParam(':location_id', $locationId);
    $stmt->execute();
    
    return $stmt->fetchAll(PDO::FETCH_ASSOC);
}

// Get all customers (for super admin)
function getAllCustomers() {
    $conn = getDbConnection();
    
    $stmt = $conn->prepare("
        SELECT c.*, l.name as location_name, lc.card_number, lc.stamp_count 
        FROM customers c
        JOIN locations l ON c.location_id = l.id
        LEFT JOIN loyalty_cards lc ON c.id = lc.customer_id
        ORDER BY c.name
    ");
    $stmt->execute();
    
    return $stmt->fetchAll(PDO::FETCH_ASSOC);
}

// Generate a slug from a string
function generateSlug($string) {
    $string = strtolower($string);
    $string = preg_replace('/[^a-z0-9\s-]/', '', $string);
    $string = preg_replace('/[\s-]+/', '-', $string);
    $string = trim($string, '-');
    return $string;
}

// Get customer by barcode
function getCustomerByBarcode($barcode) {
    $conn = getDbConnection();
    
    $stmt = $conn->prepare("
        SELECT c.*, l.name as location_name, l.reward_threshold, 
               lc.id as card_id, lc.card_number, lc.stamp_count, lc.rewards_earned
        FROM customers c
        JOIN locations l ON c.location_id = l.id
        JOIN loyalty_cards lc ON c.id = lc.customer_id
        WHERE lc.card_number = :barcode
    ");
    $stmt->bindParam(':barcode', $barcode);
    $stmt->execute();
    
    return $stmt->fetch(PDO::FETCH_ASSOC);
}

// Get customer by card ID
function getCustomerByCardId($cardId) {
    $conn = getDbConnection();
    
    $stmt = $conn->prepare("
        SELECT c.*, l.name as location_name, l.reward_threshold, 
               lc.id as card_id, lc.card_number, lc.stamp_count, lc.rewards_earned
        FROM customers c
        JOIN locations l ON c.location_id = l.id
        JOIN loyalty_cards lc ON c.id = lc.customer_id
        WHERE lc.id = :card_id
    ");
    $stmt->bindParam(':card_id', $cardId);
    $stmt->execute();
    
    return $stmt->fetch(PDO::FETCH_ASSOC);
}

/**
 * Generate default pass images if they don't exist
 */
function generateDefaultPassImages() {
    $assetsDir = __DIR__ . '/../assets/pass_images';
    
    if (!file_exists($assetsDir)) {
        mkdir($assetsDir, 0755, true);
    }
    
    // Define image sizes
    $imageSizes = [
        'icon.png' => [29, 29],
        'icon@2x.png' => [58, 58],
        'logo.png' => [160, 50],
        'logo@2x.png' => [320, 100],
        'strip.png' => [320, 84],
        'strip@2x.png' => [640, 168],
        'thumbnail.png' => [90, 90],
        'thumbnail@2x.png' => [180, 180],
    ];
    
    // Generate placeholder images
    foreach ($imageSizes as $filename => $size) {
        $imagePath = $assetsDir . '/' . $filename;
        
        if (!file_exists($imagePath)) {
            $image = imagecreatetruecolor($size[0], $size[1]);
            
            // Fill with a color
            $bgColor = imagecolorallocate($image, 67, 97, 238); // Blue color
            imagefill($image, 0, 0, $bgColor);
            
            // Add text if it's a larger image
            if ($size[0] >= 90) {
                $textColor = imagecolorallocate($image, 255, 255, 255);
                $text = "Loyalty";
                $fontSize = min(5, $size[0] / 20);
                imagestring($image, $fontSize, $size[0]/2 - strlen($text)*$fontSize, $size[1]/2 - $fontSize, $text, $textColor);
            }
            
            // Save the image
            imagepng($image, $imagePath);
            imagedestroy($image);
        }
    }
}

function getJoinUrl($slug) {
    global $config;
    
    // Hardcoded solution for dev.trimscloud.com
    if (isset($_SERVER['HTTP_HOST']) && $_SERVER['HTTP_HOST'] === 'dev.trimscloud.com') {
        // Use a direct file path instead of a directory path
        return "https://dev.trimscloud.com/customer/register.php?location=" . urlencode($slug);
    }
    
    // Use configured base URL if available
    if (isset($config['join_url_base'])) {
        return $config['join_url_base'] . urlencode($slug);
    }
    
    // Fallback to dynamic generation with direct file path
    $baseUrl = (isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] === 'on' ? "https" : "http") . "://$_SERVER[HTTP_HOST]";
    return $baseUrl . "/customer/register.php?location=" . urlencode($slug);
}

// Add this new function after getAllLocations()
function getAllLocationAdmins() {
    $conn = getDbConnection();
    
    $stmt = $conn->prepare("
        SELECT u.*, l.name as location_name 
        FROM users u
        LEFT JOIN locations l ON u.location_id = l.id
        WHERE u.role = 'location_admin'
        ORDER BY l.name, u.name
    ");
    $stmt->execute();
    
    return $stmt->fetchAll(PDO::FETCH_ASSOC);
}

// Modify the resetAdminPassword function
function resetAdminPassword($adminId, $newPassword) {
    $conn = getDbConnection();
    
    try {
        $hashedPassword = password_hash($newPassword, PASSWORD_DEFAULT);
        
        // Update the password
        $stmt = $conn->prepare("
            UPDATE users 
            SET password = :password 
            WHERE id = :id AND role = 'location_admin'
        ");
        $stmt->bindParam(':password', $hashedPassword);
        $stmt->bindParam(':id', $adminId);
        
        if ($stmt->execute()) {
            return ['success' => true];
        }
        
        return ['success' => false, 'message' => 'Failed to reset password'];
    } catch (PDOException $e) {
        return ['success' => false, 'message' => $e->getMessage()];
    }
}

// Add a new function to get the card URL
function getCardUrl($cardNumber) {
    global $config;
    
    // Hardcoded solution for dev.trimscloud.com
    if (isset($_SERVER['HTTP_HOST']) && $_SERVER['HTTP_HOST'] === 'dev.trimscloud.com') {
        return "https://dev.trimscloud.com/customer/card.php?card_number=" . urlencode($cardNumber);
    }
    
    // Fallback to dynamic generation
    $baseUrl = (isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] === 'on' ? "https" : "http") . "://$_SERVER[HTTP_HOST]";
    return $baseUrl . "/customer/card.php?card_number=" . urlencode($cardNumber);
}
?>
