<?php
/**
 * PassKit Web Service Debugging Tool
 * 
 * This tool helps diagnose issues with the PassKit web service
 */

// Enable error reporting for debugging
ini_set('display_errors', 1);
ini_set('display_startup_errors', 1);
error_reporting(E_ALL);

// Set up logging
$logDir = __DIR__ . '/../../../logs';
if (!is_dir($logDir)) {
    mkdir($logDir, 0777, true);
}
$logFile = $logDir . '/passkit_debug.log';

// Log function for debugging
function logMessage($message) {
    global $logFile;
    file_put_contents($logFile, date('Y-m-d H:i:s') . ' - ' . $message . "\n", FILE_APPEND);
}

// Include database functions
require_once __DIR__ . '/../../../includes/functions.php';

try {
    $conn = getDbConnection();
    $dbConnected = true;
} catch (Exception $e) {
    $dbConnected = false;
    logMessage("Database connection failed: " . $e->getMessage());
}

// Basic HTML header
header('Content-Type: text/html; charset=utf-8');
?>
<!DOCTYPE html>
<html>
<head>
    <title>PassKit Diagnostics</title>
    <style>
        body { font-family: Arial, sans-serif; margin: 20px; line-height: 1.6; }
        .container { max-width: 800px; margin: 0 auto; }
        .section { background: #f9f9f9; border-radius: 5px; padding: 20px; margin-bottom: 20px; box-shadow: 0 2px 4px rgba(0,0,0,0.1); }
        table { width: 100%; border-collapse: collapse; }
        th, td { padding: 8px; text-align: left; border-bottom: 1px solid #ddd; }
        th { background-color: #f2f2f2; }
        tr:hover { background-color: #f5f5f5; }
        .button { padding: 8px 15px; background: #4CAF50; color: white; border: none; border-radius: 4px; cursor: pointer; text-decoration: none; display: inline-block; }
        .button:hover { background: #45a049; }
        .success { color: green; }
        .error { color: red; }
        .warning { color: orange; }
        .code { font-family: monospace; background: #eee; padding: 10px; border-radius: 4px; white-space: pre-wrap; }
    </style>
</head>
<body>
    <div class="container">
        <h1>PassKit Diagnostics</h1>
        
        <div class="section">
            <h2>Device Registrations</h2>
            
            <?php
            if (!$dbConnected) {
                echo '<p class="error">Database connection failed. Cannot retrieve device registrations.</p>';
            } else {
                try {
                    $stmt = $conn->query("SELECT * FROM pass_registrations ORDER BY updated_at DESC LIMIT 20");
                    $registrations = $stmt->fetchAll(PDO::FETCH_ASSOC);
                    
                    if (count($registrations) > 0) {
                        echo '<table>';
                        echo '<tr><th>ID</th><th>Device ID</th><th>Pass Type</th><th>Serial Number</th><th>Created</th><th>Updated</th></tr>';
                        
                        foreach ($registrations as $reg) {
                            echo '<tr>';
                            echo '<td>' . htmlspecialchars($reg['id']) . '</td>';
                            echo '<td>' . htmlspecialchars(substr($reg['device_library_identifier'], 0, 10) . '...') . '</td>';
                            echo '<td>' . htmlspecialchars($reg['pass_type_identifier']) . '</td>';
                            echo '<td>' . htmlspecialchars($reg['serial_number']) . '</td>';
                            echo '<td>' . htmlspecialchars($reg['created_at']) . '</td>';
                            echo '<td>' . htmlspecialchars($reg['updated_at']) . '</td>';
                            echo '</tr>';
                        }
                        
                        echo '</table>';
                    } else {
                        echo '<p class="warning">No device registrations found in the database.</p>';
                    }
                } catch (PDOException $e) {
                    echo '<p class="error">Error retrieving registrations: ' . htmlspecialchars($e->getMessage()) . '</p>';
                }
            }
            ?>
        </div>
        
        <div class="section">
            <h2>Pass Updates</h2>
            
            <?php
            if (!$dbConnected) {
                echo '<p class="error">Database connection failed. Cannot retrieve pass updates.</p>';
            } else {
                try {
                    $stmt = $conn->query("SELECT * FROM pass_updates ORDER BY last_updated DESC LIMIT 20");
                    $updates = $stmt->fetchAll(PDO::FETCH_ASSOC);
                    
                    if (count($updates) > 0) {
                        echo '<table>';
                        echo '<tr><th>Pass ID</th><th>Last Updated</th><th>Last Pushed</th><th>Version</th></tr>';
                        
                        foreach ($updates as $update) {
                            echo '<tr>';
                            echo '<td>' . htmlspecialchars($update['pass_id']) . '</td>';
                            echo '<td>' . htmlspecialchars($update['last_updated']) . '</td>';
                            echo '<td>' . htmlspecialchars($update['last_pushed'] ?: 'Never') . '</td>';
                            echo '<td>' . htmlspecialchars($update['version']) . '</td>';
                            echo '</tr>';
                        }
                        
                        echo '</table>';
                    } else {
                        echo '<p class="warning">No pass updates found in the database.</p>';
                    }
                } catch (PDOException $e) {
                    echo '<p class="error">Error retrieving pass updates: ' . htmlspecialchars($e->getMessage()) . '</p>';
                }
            }
            ?>
        </div>
        
        <div class="section">
            <h2>API Endpoints</h2>
            <p>Here are the key API endpoints for Apple Wallet passes:</p>
            
            <div class="code">
1. Register Device:
   POST https://dev.trimscloud.com/webservice/v1/passes/pass.pointify.pk/{serialNumber}/register
   
2. Unregister Device:
   DELETE https://dev.trimscloud.com/webservice/v1/devices/{deviceLibraryIdentifier}/registrations/pass.pointify.pk/{serialNumber}
   
3. Get Serial Numbers:
   GET https://dev.trimscloud.com/webservice/v1/devices/{deviceLibraryIdentifier}/registrations/pass.pointify.pk
   
4. Get Latest Pass:
   GET https://dev.trimscloud.com/webservice/v1/passes/pass.pointify.pk/{serialNumber}
   
5. Log:
   POST https://dev.trimscloud.com/webservice/v1/passes/v1/log
            </div>
        </div>
        
        <div class="section">
            <h2>Admin Actions</h2>
            <p>Use these tools to manage and update passes:</p>
            
            <ul>
                <li><a href="update_card.php" class="button">Update Card</a> - Add stamps and send push notifications</li>
                <li><a href="update_card.php?card=LC202503233546&stamps=5" class="button">Update Demo Card (5 stamps)</a></li>
                <li><a href="update_card.php?card=LC202503233546&increase=1" class="button">Add 1 Stamp to Demo Card</a></li>
            </ul>
        </div>
        
        <div class="section">
            <h2>Test Card Registration</h2>
            
            <?php if (!$dbConnected): ?>
                <p class="error">Database connection failed. Cannot perform test registration.</p>
            <?php else: ?>
            <form method="post" action="">
                <p>
                    <label>Card Number:</label>
                    <input type="text" name="card_number" value="<?php echo isset($_POST['card_number']) ? htmlspecialchars($_POST['card_number']) : 'LC202503233546'; ?>" required>
                </p>
                <p>
                    <label>Device ID:</label>
                    <input type="text" name="device_id" value="<?php echo isset($_POST['device_id']) ? htmlspecialchars($_POST['device_id']) : 'test_device_' . bin2hex(random_bytes(4)); ?>" required>
                </p>
                <p>
                    <label>Push Token:</label>
                    <input type="text" name="push_token" value="<?php echo isset($_POST['push_token']) ? htmlspecialchars($_POST['push_token']) : bin2hex(random_bytes(16)); ?>" required>
                </p>
                <p>
                    <button type="submit" name="register" class="button">Register Device</button>
                </p>
            </form>
            
            <?php
            // Handle test registration
            if (isset($_POST['register']) && !empty($_POST['card_number']) && !empty($_POST['device_id']) && !empty($_POST['push_token'])) {
                $serialNumber = $_POST['card_number'];
                $deviceId = $_POST['device_id'];
                $pushToken = $_POST['push_token'];
                $passTypeId = 'pass.pointify.pk';
                
                try {
                    // Check if this device is already registered
                    $stmt = $conn->prepare("
                        SELECT COUNT(*) FROM pass_registrations 
                        WHERE serial_number = ? AND device_library_identifier = ? AND pass_type_identifier = ?
                    ");
                    $stmt->execute([$serialNumber, $deviceId, $passTypeId]);
                    $exists = $stmt->fetchColumn() > 0;
                    
                    if ($exists) {
                        // Update the registration
                        $stmt = $conn->prepare("
                            UPDATE pass_registrations 
                            SET push_token = ?,
                                updated_at = NOW()
                            WHERE serial_number = ? AND device_library_identifier = ? AND pass_type_identifier = ?
                        ");
                        $stmt->execute([$pushToken, $serialNumber, $deviceId, $passTypeId]);
                        echo '<p class="success">✓ Updated existing device registration</p>';
                    } else {
                        // Create a new registration
                        $stmt = $conn->prepare("
                            INSERT INTO pass_registrations (
                                serial_number, 
                                device_library_identifier, 
                                pass_type_identifier,
                                pass_type_id,
                                push_token
                            ) VALUES (?, ?, ?, ?, ?)
                        ");
                        $stmt->execute([
                            $serialNumber,
                            $deviceId,
                            $passTypeId,
                            $passTypeId,
                            $pushToken
                        ]);
                        echo '<p class="success">✓ Created new device registration</p>';
                        
                        // Make sure we have an entry in the pass_updates table
                        $stmt = $conn->prepare("
                            INSERT IGNORE INTO pass_updates (pass_id, last_updated) VALUES (?, NOW())
                        ");
                        $stmt->execute([$serialNumber]);
                    }
                } catch (PDOException $e) {
                    echo '<p class="error">Error creating registration: ' . htmlspecialchars($e->getMessage()) . '</p>';
                }
            }
            ?>
            <?php endif; ?>
        </div>
        
        <div class="section">
            <h2>Request Log</h2>
            <?php
            // Show the last 20 lines of the log file
            $logFile = $logDir . '/passkit_webservice.log';
            
            if (file_exists($logFile)) {
                $logContent = file($logFile, FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES);
                if (!empty($logContent)) {
                    $logContent = array_slice($logContent, -20);
                    echo '<div class="code">' . htmlspecialchars(implode("\n", $logContent)) . '</div>';
                } else {
                    echo '<p class="warning">Log file exists but is empty.</p>';
                }
            } else {
                echo '<p class="warning">Log file not found at: ' . htmlspecialchars($logFile) . '</p>';
            }
            ?>
        </div>
    </div>
</body>
</html> 