<?php
/**
 * PassKit Web Service
 * 
 * This file handles all PassKit web service API requests for pass updates
 * 
 * Required endpoints by Apple:
 * - POST /v1/passes/{passTypeIdentifier}/{serialNumber}/register - Register a device to receive push notifications for a pass
 * - DELETE /v1/passes/{passTypeIdentifier}/{serialNumber}/devices/{deviceLibraryIdentifier} - Unregister a device
 * - GET /v1/passes/{passTypeIdentifier}/{serialNumber} - Get the latest version of a pass
 */

// Enable error reporting for debugging
ini_set('display_errors', 1);
ini_set('display_startup_errors', 1);
error_reporting(E_ALL);

// Set up logging
$logDir = __DIR__ . '/../../../logs';
if (!is_dir($logDir)) {
    mkdir($logDir, 0777, true);
}
$logFile = $logDir . '/passkit_webservice.log';

// Log function for debugging
function logMessage($message) {
    global $logFile;
    file_put_contents($logFile, date('Y-m-d H:i:s') . ' - ' . $message . "\n", FILE_APPEND);
}

// Log all incoming requests
logMessage('REQUEST: ' . $_SERVER['REQUEST_METHOD'] . ' ' . $_SERVER['REQUEST_URI']);
logMessage('HEADERS: ' . json_encode(getallheaders()));

// Include database functions
require_once __DIR__ . '/../../../includes/functions.php';
$conn = getDbConnection();

// Handle authorization
$authToken = extractAuthToken();
if (!empty($authToken)) {
    logMessage('Auth token provided: ' . substr($authToken, 0, 10) . '...');
} else {
    logMessage('No auth token provided');
}

// Extract the request path manually
$requestUri = $_SERVER['REQUEST_URI'] ?? '';
logMessage('Full request URI: ' . $requestUri);

// Parse for double v1 prefix pattern
if (strpos($requestUri, '/webservice/v1/passes/v1/') !== false) {
    // This is a device trying to access our API with the double v1 prefix
    $urlParts = explode('/webservice/v1/passes/v1/', $requestUri);
    if (count($urlParts) > 1) {
        $pathParts = explode('/', trim($urlParts[1], '?'));
        logMessage('Path parts from double v1 format: ' . json_encode($pathParts));
        
        // Match pattern: devices/{deviceId}/registrations/{passTypeId}/{serialNumber}
        if (count($pathParts) >= 5 && $pathParts[0] == 'devices' && $pathParts[2] == 'registrations') {
            $deviceId = $pathParts[1];
            $passTypeId = $pathParts[3];
            $serialNumber = $pathParts[4];
            
            // Handle based on HTTP method
            if ($_SERVER['REQUEST_METHOD'] === 'POST') {
                // Registration attempt
                logMessage("Detected device registration with double v1 prefix: Device=$deviceId, PassType=$passTypeId, Serial=$serialNumber");
                
                // Get the request body and log it for debugging
                $requestBody = file_get_contents('php://input');
                logMessage("Registration request body: " . $requestBody);
                
                // Parse JSON data
                $data = json_decode($requestBody, true);
                
                // Check if we got a valid pushToken
                if (!isset($data['pushToken']) || empty($data['pushToken'])) {
                    logMessage("Error: Missing or empty push token in request body");
                    http_response_code(400);
                    echo json_encode(['status' => 'error', 'message' => 'Missing push token']);
                    exit;
                }
                
                // Extract push token from request body
                $pushToken = $data['pushToken'];
                logMessage("Found push token: " . substr($pushToken, 0, 10) . "...");
                
                try {
                    // Verify the card exists before registering
                    $stmt = $conn->prepare("SELECT COUNT(*) FROM loyalty_cards WHERE card_number = ?");
                    $stmt->execute([$serialNumber]);
                    $cardExists = $stmt->fetchColumn() > 0;
                    
                    if (!$cardExists) {
                        logMessage("Error: Loyalty card not found for serial number: $serialNumber");
                        http_response_code(404);
                        echo json_encode(['status' => 'error', 'message' => 'Pass not found']);
                        exit;
                    }
                    
                    // First, check if this device is already registered for this pass
                    $stmt = $conn->prepare("
                        SELECT COUNT(*) FROM pass_registrations 
                        WHERE serial_number = ? AND device_library_identifier = ? AND pass_type_identifier = ?
                    ");
                    $stmt->execute([$serialNumber, $deviceId, $passTypeId]);
                    $exists = $stmt->fetchColumn() > 0;
                    
                    if ($exists) {
                        // Update the push token
                        $stmt = $conn->prepare("
                            UPDATE pass_registrations 
                            SET push_token = ?,
                                updated_at = NOW()
                            WHERE serial_number = ? AND device_library_identifier = ? AND pass_type_identifier = ?
                        ");
                        $stmt->execute([$pushToken, $serialNumber, $deviceId, $passTypeId]);
                        logMessage("Updated existing registration for device $deviceId");
                    } else {
                        // Create a new registration
                        $stmt = $conn->prepare("
                            INSERT INTO pass_registrations (
                                serial_number, 
                                device_library_identifier, 
                                pass_type_identifier,
                                pass_type_id,
                                push_token
                            ) VALUES (?, ?, ?, ?, ?)
                        ");
                        $result = $stmt->execute([
                            $serialNumber,
                            $deviceId,
                            $passTypeId,
                            $passTypeId,
                            $pushToken
                        ]);
                        
                        if (!$result) {
                            logMessage("Error creating registration: " . print_r($stmt->errorInfo(), true));
                            http_response_code(500);
                            echo json_encode(['status' => 'error', 'message' => 'Database error during registration']);
                            exit;
                        }
                        
                        logMessage("Created new registration for device $deviceId");
                        
                        // Make sure we have an entry in the pass_updates table
                        $stmt = $conn->prepare("
                            INSERT IGNORE INTO pass_updates (pass_id, last_updated) VALUES (?, NOW())
                        ");
                        $stmt->execute([$serialNumber]);
                    }
                    
                    // Return success
                    http_response_code(201); // Created
                    echo json_encode(['status' => 'success']);
                } catch (Exception $e) {
                    http_response_code(500);
                    echo json_encode(['status' => 'error', 'message' => 'Database error: ' . $e->getMessage()]);
                    logMessage('Database error during registration: ' . $e->getMessage());
                }
                exit;
            } 
            else if ($_SERVER['REQUEST_METHOD'] === 'DELETE') {
                // Unregistration attempt
                logMessage("Detected device unregistration with double v1 prefix: Device=$deviceId, PassType=$passTypeId, Serial=$serialNumber");
                
                try {
                    // Delete the registration
                    $stmt = $conn->prepare("
                        DELETE FROM pass_registrations 
                        WHERE serial_number = ? AND device_library_identifier = ? AND pass_type_identifier = ?
                    ");
                    $stmt->execute([$serialNumber, $deviceId, $passTypeId]);
                    
                    // Check if any row was affected
                    if ($stmt->rowCount() > 0) {
                        http_response_code(200);
                        echo json_encode(['status' => 'success']);
                        logMessage("Device $deviceId unregistered successfully");
                    } else {
                        http_response_code(404);
                        echo json_encode(['status' => 'error', 'message' => 'Registration not found']);
                        logMessage("Error: Registration not found for device $deviceId");
                    }
                } catch (Exception $e) {
                    http_response_code(500);
                    echo json_encode(['status' => 'error', 'message' => 'Database error: ' . $e->getMessage()]);
                    logMessage('Database error during unregistration: ' . $e->getMessage());
                }
                exit;
            }
            else {
                // Invalid HTTP method for this endpoint
                logMessage("Error: Invalid HTTP method " . $_SERVER['REQUEST_METHOD'] . " for device registration/unregistration");
                
                // If this is a browser GET request, provide help information
                $isBrowser = isset($_SERVER['HTTP_USER_AGENT']) && 
                            (strpos($_SERVER['HTTP_USER_AGENT'], 'Mozilla') !== false || 
                             strpos($_SERVER['HTTP_USER_AGENT'], 'Chrome') !== false || 
                             strpos($_SERVER['HTTP_USER_AGENT'], 'Safari') !== false || 
                             strpos($_SERVER['HTTP_USER_AGENT'], 'Firefox') !== false);
                
                if ($_SERVER['REQUEST_METHOD'] === 'GET' && $isBrowser) {
                    header('Content-Type: text/html');
                    echo '<html><head><title>Apple Wallet PassKit API Endpoint</title>';
                    echo '<style>body{font-family:Arial,sans-serif;margin:20px;line-height:1.6;max-width:800px;margin:0 auto;padding:20px}
                          h1{color:#333}pre{background:#f5f5f5;padding:10px;border-radius:5px;overflow-x:auto}
                          .note{background:#fffacd;padding:10px;border-left:4px solid #ffcc00;margin:20px 0}</style>';
                    echo '</head><body>';
                    echo '<h1>PassKit API Endpoint</h1>';
                    echo '<p>This is a PassKit web service endpoint for Apple Wallet passes. It is designed to be accessed by the Wallet app, not directly through a browser.</p>';
                    echo '<div class="note"><strong>Note:</strong> You\'re seeing this message because you accessed this endpoint with a GET request from a browser. 
                          The registration endpoint requires a POST request with a valid push token in the request body.</div>';
                    echo '<h2>Endpoint Information</h2>';
                    echo '<ul>';
                    echo '<li><strong>URL:</strong> ' . htmlspecialchars($_SERVER['REQUEST_URI']) . '</li>';
                    echo '<li><strong>Device ID:</strong> ' . htmlspecialchars($deviceId) . '</li>';
                    echo '<li><strong>Pass Type:</strong> ' . htmlspecialchars($passTypeId) . '</li>';
                    echo '<li><strong>Serial Number:</strong> ' . htmlspecialchars($serialNumber) . '</li>';
                    echo '</ul>';
                    echo '<h2>Valid Request Format</h2>';
                    echo '<pre>POST ' . htmlspecialchars($_SERVER['REQUEST_URI']) . "\n";
                    echo "Content-Type: application/json\n\n";
                    echo '{\n    "pushToken": "device_push_token_here"\n}</pre>';
                    echo '<p>This endpoint is part of the Apple PassKit Web Service API for managing passes in Apple Wallet.</p>';
                    echo '</body></html>';
                } else {
                    http_response_code(405); // Method Not Allowed
                    header('Allow: POST, DELETE');
                    echo json_encode(['status' => 'error', 'message' => 'Invalid HTTP method, use POST to register or DELETE to unregister']);
                }
                exit;
            }
        }
        
        // Match get serial numbers pattern: devices/{deviceId}/registrations/{passTypeId}
        if ($_SERVER['REQUEST_METHOD'] === 'GET' && count($pathParts) >= 4 && $pathParts[0] == 'devices' && $pathParts[2] == 'registrations') {
            $action = 'getSerialNumbers'; // This is a request for pass serial numbers
            $deviceId = $pathParts[1];
            $passTypeId = $pathParts[3];
            
            logMessage("Detected get serial numbers with double v1 prefix: Device=$deviceId, PassType=$passTypeId");
            getSerialNumbersForDevice($conn, $deviceId, $passTypeId);
            exit;
        }
    }
}

// Parse the URL to get pass info
$urlParts = explode('/', trim($_SERVER['PATH_INFO'] ?? '', '/'));
logMessage('URL parts: ' . json_encode($urlParts));

// Check if we're using the new URL pattern via GET parameters
$action = $_GET['action'] ?? '';
$deviceId = $_GET['device_id'] ?? '';
$passTypeId = $_GET['pass_type_id'] ?? 'pass.pointify.pk'; // Default to our pass type
$serialNumber = $_GET['serial_number'] ?? '';

// Make these variables globally accessible
global $action, $deviceId, $passTypeId, $serialNumber;

// Log for debugging
if (!empty($action)) {
    logMessage("Action: $action, Device: $deviceId, PassType: $passTypeId, Serial: $serialNumber");
}

// Default response headers
header('Content-Type: application/json');

try {
    // Create tables if they don't exist
    createTablesIfNeeded($conn);
    
    // Handle direct action parameters from htaccess rewrite
    if ($action == 'register' && !empty($deviceId) && !empty($serialNumber) && !empty($passTypeId) && $_SERVER['REQUEST_METHOD'] === 'POST') {
        logMessage("Registration request via direct params: Device=$deviceId, PassType=$passTypeId, Serial=$serialNumber");
        
        // Process device registration with the params
        registerDeviceDirect($conn, $serialNumber, $deviceId);
        exit;
    }
    
    // Handle unregister action from URL parameters (DELETE request)
    if ($action == 'unregister' && !empty($deviceId) && !empty($serialNumber) && $_SERVER['REQUEST_METHOD'] === 'DELETE') {
        logMessage("Unregister request via direct params: Device=$deviceId, PassType=$passTypeId, Serial=$serialNumber");
        
        // Process the unregister request
        unregisterDeviceDirect($conn, $serialNumber, $deviceId);
        exit;
    }
    
    // Handle getSerialNumbers action from URL parameters
    if ($action == 'getSerialNumbers' && !empty($deviceId) && !empty($passTypeId)) {
        logMessage("Get serial numbers request: Device=$deviceId, PassType=$passTypeId");
        
        // Process the get serial numbers request
        getSerialNumbersForDevice($conn, $deviceId, $passTypeId);
        exit;
    }
    
    // Handle getLatestPass action from URL parameters
    if ($action == 'getLatestPass' && !empty($passTypeId) && !empty($serialNumber)) {
        logMessage("Get latest pass request via direct params: PassType=$passTypeId, Serial=$serialNumber");
        
        // Process the get latest pass request
        getLatestPass($conn, $serialNumber);
        exit;
    }
    
    // Handle log action
    if ($action == 'log') {
        logMessage("Log request received: " . file_get_contents('php://input'));
        http_response_code(200);
        echo json_encode(['status' => 'success']);
        exit;
    }
    
    // Basic validation for standard URL pattern
    if (count($urlParts) < 2) {
        http_response_code(404);
        echo json_encode(['status' => 'error', 'message' => 'Invalid endpoint']);
        logMessage('Error: Invalid endpoint - not enough URL parts');
        exit;
    }
    
    $passTypeIdentifier = $urlParts[0];
    $serialNumber = $urlParts[1]; // This is our card number
    
    logMessage("PassTypeID: $passTypeIdentifier, SerialNumber: $serialNumber");
    
    // Verify the pass exists and the authentication token is valid
    if (!empty($authToken)) {
        if (!verifyPassAuth($conn, $serialNumber, $authToken)) {
            http_response_code(401);
            echo json_encode(['status' => 'error', 'message' => 'Invalid authentication token']);
            logMessage('Error: Invalid authentication token');
            exit;
        }
    }
    
    // Route the request based on the HTTP method and URL
    if ($_SERVER['REQUEST_METHOD'] === 'POST' && count($urlParts) === 3 && $urlParts[2] === 'register') {
        // Register a device for push notifications
        registerDevice($conn, $serialNumber);
    } 
    elseif ($_SERVER['REQUEST_METHOD'] === 'DELETE' && count($urlParts) === 4 && $urlParts[2] === 'devices') {
        // Unregister a device
        $deviceLibraryIdentifier = $urlParts[3];
        unregisterDevice($conn, $serialNumber, $deviceLibraryIdentifier);
    }
    elseif ($_SERVER['REQUEST_METHOD'] === 'GET' && count($urlParts) === 2) {
        // Get the latest version of a pass
        getLatestPass($conn, $serialNumber);
    }
    else {
        http_response_code(404);
        echo json_encode(['status' => 'error', 'message' => 'Endpoint not found']);
        logMessage('Error: Endpoint not found - ' . $_SERVER['REQUEST_METHOD'] . ' ' . $_SERVER['REQUEST_URI']);
    }
} catch (Exception $e) {
    http_response_code(500);
    echo json_encode(['status' => 'error', 'message' => 'Server error: ' . $e->getMessage()]);
    logMessage('Exception: ' . $e->getMessage());
}

/**
 * Create necessary database tables if they don't exist
 */
function createTablesIfNeeded($conn) {
    // Table for pass registrations (device registrations)
    $conn->exec("CREATE TABLE IF NOT EXISTS pass_registrations (
        id INT(11) UNSIGNED NOT NULL AUTO_INCREMENT,
        device_library_identifier VARCHAR(64) NOT NULL,
        pass_type_identifier VARCHAR(64) NOT NULL,
        pass_type_id VARCHAR(64) NOT NULL,
        serial_number VARCHAR(64) NOT NULL,
        push_token VARCHAR(255) NOT NULL,
        created_at TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP,
        updated_at TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP,
        PRIMARY KEY (id),
        UNIQUE KEY device_pass (device_library_identifier, pass_type_identifier, serial_number)
    )");
    
    // Table for tracking pass updates
    $conn->exec("CREATE TABLE IF NOT EXISTS pass_updates (
        pass_id VARCHAR(50) PRIMARY KEY,
        last_updated TIMESTAMP DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP,
        last_pushed TIMESTAMP NULL,
        version INT DEFAULT 1
    )");
    
    logMessage('Database tables verified');
}

/**
 * Verify pass authentication token
 */
function verifyPassAuth($conn, $serialNumber, $authToken) {
    // Query to check if the pass exists with this auth token
    $stmt = $conn->prepare("
        SELECT COUNT(*) FROM loyalty_cards 
        WHERE card_number = ? AND auth_token = ?
    ");
    
    $stmt->execute([$serialNumber, $authToken]);
    $count = $stmt->fetchColumn();
    
    logMessage("Auth check for pass $serialNumber: " . ($count > 0 ? 'valid' : 'invalid'));
    
    return $count > 0;
}

/**
 * Register a device to receive push notifications for a pass
 */
function registerDevice($conn, $serialNumber) {
    // Parse the request body as JSON
    $requestBody = file_get_contents('php://input');
    logMessage("Registration request body: " . $requestBody);
    
    $data = json_decode($requestBody, true);
    
    // Validate required fields
    if (!isset($data['pushToken']) || !isset($data['deviceLibraryIdentifier'])) {
        http_response_code(400);
        echo json_encode(['status' => 'error', 'message' => 'Missing required fields']);
        logMessage('Error: Missing required fields for registration. Required: pushToken and deviceLibraryIdentifier');
        return;
    }
    
    $deviceLibraryIdentifier = $data['deviceLibraryIdentifier'];
    $pushToken = $data['pushToken'];
    $passTypeIdentifier = isset($data['passTypeIdentifier']) ? $data['passTypeIdentifier'] : 'pass.pointify.pk';
    
    logMessage("Registering device $deviceLibraryIdentifier for pass $serialNumber with token " . substr($pushToken, 0, 10) . "... (Pass Type: $passTypeIdentifier)");
    
    try {
        // Verify the card exists before registering
        $stmt = $conn->prepare("SELECT COUNT(*) FROM loyalty_cards WHERE card_number = ?");
        $stmt->execute([$serialNumber]);
        $cardExists = $stmt->fetchColumn() > 0;
        
        if (!$cardExists) {
            logMessage("Error: Loyalty card not found for serial number: $serialNumber");
            http_response_code(404);
            echo json_encode(['status' => 'error', 'message' => 'Pass not found']);
            return;
        }
        
        // First, check if this device is already registered for this pass
        $stmt = $conn->prepare("
            SELECT COUNT(*) FROM pass_registrations 
            WHERE serial_number = ? AND device_library_identifier = ? AND pass_type_identifier = ?
        ");
        $stmt->execute([$serialNumber, $deviceLibraryIdentifier, $passTypeIdentifier]);
        $exists = $stmt->fetchColumn() > 0;
        
        if ($exists) {
            // Update the push token
            $stmt = $conn->prepare("
                UPDATE pass_registrations 
                SET push_token = ?,
                    updated_at = NOW()
                WHERE serial_number = ? AND device_library_identifier = ? AND pass_type_identifier = ?
            ");
            $stmt->execute([$pushToken, $serialNumber, $deviceLibraryIdentifier, $passTypeIdentifier]);
            logMessage("Updated existing registration for device $deviceLibraryIdentifier");
        } else {
            // Create a new registration
            $stmt = $conn->prepare("
                INSERT INTO pass_registrations (
                    serial_number, 
                    device_library_identifier, 
                    pass_type_identifier,
                    pass_type_id,
                    push_token
                ) VALUES (?, ?, ?, ?, ?)
            ");
            $result = $stmt->execute([
                $serialNumber,
                $deviceLibraryIdentifier,
                $passTypeIdentifier,
                $passTypeIdentifier, // We're using the same value for both fields for now
                $pushToken
            ]);
            
            if (!$result) {
                logMessage("Error creating registration: " . print_r($stmt->errorInfo(), true));
                http_response_code(500);
                echo json_encode(['status' => 'error', 'message' => 'Database error during registration']);
                return;
            }
            
            logMessage("Created new registration for device $deviceLibraryIdentifier");
            
            // Make sure we have an entry in the pass_updates table
            $stmt = $conn->prepare("
                INSERT IGNORE INTO pass_updates (pass_id, last_updated) VALUES (?, NOW())
            ");
            $stmt->execute([$serialNumber]);
        }
        
        // Return success
        http_response_code(201); // Created
        echo json_encode(['status' => 'success']);
    } catch (Exception $e) {
        http_response_code(500);
        echo json_encode(['status' => 'error', 'message' => 'Database error: ' . $e->getMessage()]);
        logMessage('Database error during registration: ' . $e->getMessage());
    }
}

/**
 * Unregister a device
 */
function unregisterDevice($conn, $serialNumber, $deviceLibraryIdentifier) {
    // Get the passTypeIdentifier from the request parameters if available
    $passTypeIdentifier = isset($_GET['pass_type_id']) ? $_GET['pass_type_id'] : 'pass.pointify.pk';
    
    logMessage("Unregistering device $deviceLibraryIdentifier from pass $serialNumber with type $passTypeIdentifier");
    
    try {
        // Delete the registration with pass type identifier
        $stmt = $conn->prepare("
            DELETE FROM pass_registrations 
            WHERE serial_number = ? AND device_library_identifier = ? AND pass_type_identifier = ?
        ");
        $stmt->execute([$serialNumber, $deviceLibraryIdentifier, $passTypeIdentifier]);
        
        // Check if any row was affected
        if ($stmt->rowCount() > 0) {
            http_response_code(200);
            echo json_encode(['status' => 'success']);
            logMessage("Device $deviceLibraryIdentifier unregistered successfully");
        } else {
            http_response_code(404);
            echo json_encode(['status' => 'error', 'message' => 'Registration not found']);
            logMessage("Error: Registration not found for device $deviceLibraryIdentifier");
        }
    } catch (Exception $e) {
        http_response_code(500);
        echo json_encode(['status' => 'error', 'message' => 'Database error: ' . $e->getMessage()]);
        logMessage('Database error during unregistration: ' . $e->getMessage());
    }
}

/**
 * Get the latest version of a pass
 */
function getLatestPass($conn, $serialNumber) {
    global $passTypeId; // Use the one from URL parameters
    
    logMessage("Getting latest pass for: $serialNumber (Pass Type: $passTypeId)");
    
    // If no serial number was provided directly, try to extract from URL
    if (empty($serialNumber) && !empty($_GET['serial_number'])) {
        $serialNumber = $_GET['serial_number'];
        logMessage("Using serial number from query params: $serialNumber");
    }
    
    // Get the authentication token
    $authToken = extractAuthToken();
    logMessage("Authentication token for pass retrieval: " . (empty($authToken) ? 'None provided' : 'Token provided'));
    
    // If auth token is provided, verify it
    if (!empty($authToken) && !verifyPassAuth($conn, $serialNumber, $authToken)) {
        http_response_code(401);
        echo json_encode(['status' => 'error', 'message' => 'Invalid authentication token']);
        logMessage('Error: Invalid authentication token for pass retrieval');
        return;
    }
    
    // First, check if the pass exists in the database
    $stmt = $conn->prepare("SELECT * FROM loyalty_cards WHERE card_number = ?");
    $stmt->execute([$serialNumber]);
    $card = $stmt->fetch(PDO::FETCH_ASSOC);
    
    if (!$card) {
        http_response_code(404);
        echo json_encode(['status' => 'error', 'message' => 'Pass not found']);
        logMessage("Error: Pass not found for serial number: $serialNumber");
        return;
    }
    
    // Check if there's a newer version in the pass_updates table
    $ifModifiedSince = isset($_SERVER['HTTP_IF_MODIFIED_SINCE']) ? strtotime($_SERVER['HTTP_IF_MODIFIED_SINCE']) : 0;
    logMessage("If-Modified-Since header: " . ($_SERVER['HTTP_IF_MODIFIED_SINCE'] ?? 'Not provided'));
    
    $stmt = $conn->prepare("SELECT last_updated FROM pass_updates WHERE pass_id = ?");
    $stmt->execute([$serialNumber]);
    $updateInfo = $stmt->fetch(PDO::FETCH_ASSOC);
    
    if ($updateInfo) {
        $lastUpdated = strtotime($updateInfo['last_updated']);
        logMessage("Last update timestamp for pass: " . date('Y-m-d H:i:s', $lastUpdated));
        
        // If client provides If-Modified-Since header and the pass hasn't been updated since then
        if ($ifModifiedSince && $lastUpdated <= $ifModifiedSince) {
            http_response_code(304); // Not Modified
            logMessage("Pass not modified since: " . date('Y-m-d H:i:s', $ifModifiedSince) . ", returning 304");
            return;
        }
    }
    
    // Generate a new pass with the current data
    try {
        $passFilePath = generatePass($card);
        
        if (!file_exists($passFilePath)) {
            http_response_code(500);
            echo json_encode(['status' => 'error', 'message' => 'Failed to generate the pass']);
            logMessage("Error: Failed to generate pass file at $passFilePath");
            return;
        }
        
        // Set the Last-Modified header
        if ($updateInfo) {
            $lastModified = gmdate('D, d M Y H:i:s', strtotime($updateInfo['last_updated'])) . ' GMT';
            header('Last-Modified: ' . $lastModified);
            logMessage("Setting Last-Modified header: " . $lastModified);
        } else {
            // Even if we don't have update info, we still need to send a Last-Modified header
            $lastModified = gmdate('D, d M Y H:i:s') . ' GMT';
            header('Last-Modified: ' . $lastModified);
            logMessage("Setting default Last-Modified header: " . $lastModified);
        }
        
        // Set content type for .pkpass file
        header('Content-Type: application/vnd.apple.pkpass');
        header('Content-Disposition: attachment; filename="' . basename($passFilePath) . '"');
        header('Content-Length: ' . filesize($passFilePath));
        
        // Output the file
        readfile($passFilePath);
        logMessage("Successfully sent pass file: $passFilePath");
    } catch (Exception $e) {
        http_response_code(500);
        echo json_encode(['status' => 'error', 'message' => 'Error generating pass: ' . $e->getMessage()]);
        logMessage("Error generating pass: " . $e->getMessage());
    }
}

/**
 * Generate a pass file for a card
 * 
 * This function generates a complete .pkpass file
 */
function generatePass($card) {
    // We'll reuse the existing pass generation logic
    $cardNumber = $card['card_number'];
    
    // Generate the pass and get the file path
    $passFilePath = __DIR__ . '/../../../passes/' . $cardNumber . '.pkpass';
    $passDir = dirname($passFilePath);
    
    // Make sure the directory exists
    if (!is_dir($passDir)) {
        mkdir($passDir, 0777, true);
    }
    
    // Check if the file already exists and is recent (less than 5 minutes old)
    $fileExists = file_exists($passFilePath);
    $fileRecent = $fileExists && (time() - filemtime($passFilePath) < 300);
    
    if (!$fileExists || !$fileRecent) {
        logMessage("Pass file doesn't exist or is outdated, generating new one: " . $passFilePath);
        $passGeneratorURL = 'https://dev.trimscloud.com/customer/generate_pass.php?card_number=' . urlencode($cardNumber) . '&api=1';
        
        // We'll make a curl request to the existing generator
        $ch = curl_init($passGeneratorURL);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false); // For self-signed certs
        $response = curl_exec($ch);
        
        if ($response === false) {
            logMessage("cURL error: " . curl_error($ch));
        } else {
            logMessage("Successfully generated pass via API call: " . substr($response, 0, 100));
        }
        
        curl_close($ch);
        
        // Check if the file exists now
        if (!file_exists($passFilePath)) {
            logMessage("Pass file not found after API call, trying to generate it directly: " . $passFilePath);
            
            // If the pass file doesn't exist, try generating it directly
            // Store current directory to return to it after generating the pass
            $currentDir = getcwd();
            chdir(__DIR__ . '/../../../customer/');
            
            // Generate pass directly (simplified version for fallback)
            include_once(__DIR__ . '/../../../customer/generate_pass.php');
            
            // Return to original directory
            chdir($currentDir);
        }
    } else {
        logMessage("Using existing pass file: " . $passFilePath);
    }
    
    // Final verification
    if (!file_exists($passFilePath)) {
        throw new Exception("Failed to generate pass file: " . $passFilePath);
    }
    
    return $passFilePath;
}

/**
 * Register a device directly using the parameters from URL rewrite
 */
function registerDeviceDirect($conn, $serialNumber, $deviceLibraryIdentifier) {
    // Parse the request body as JSON
    $requestBody = file_get_contents('php://input');
    logMessage("Registration request body: " . $requestBody);
    
    $data = json_decode($requestBody, true);
    
    // Apple sends the push token in the request body
    if (!isset($data['pushToken']) || empty($data['pushToken'])) {
        http_response_code(400);
        echo json_encode(['status' => 'error', 'message' => 'Missing push token']);
        logMessage('Error: Missing or empty push token for registration');
        return;
    }
    
    $pushToken = $data['pushToken'];
    $passTypeIdentifier = isset($_GET['pass_type_id']) ? $_GET['pass_type_id'] : 'pass.pointify.pk';
    
    logMessage("Directly registering device $deviceLibraryIdentifier for pass $serialNumber with token " . substr($pushToken, 0, 10) . "... (Pass Type: $passTypeIdentifier)");
    
    try {
        // Verify the card exists before registering
        $stmt = $conn->prepare("SELECT COUNT(*) FROM loyalty_cards WHERE card_number = ?");
        $stmt->execute([$serialNumber]);
        $cardExists = $stmt->fetchColumn() > 0;
        
        if (!$cardExists) {
            logMessage("Error: Loyalty card not found for serial number: $serialNumber");
            http_response_code(404);
            echo json_encode(['status' => 'error', 'message' => 'Pass not found']);
            return;
        }
        
        // First, check if this device is already registered for this pass
        $stmt = $conn->prepare("
            SELECT COUNT(*) FROM pass_registrations 
            WHERE serial_number = ? AND device_library_identifier = ? AND pass_type_identifier = ?
        ");
        $stmt->execute([$serialNumber, $deviceLibraryIdentifier, $passTypeIdentifier]);
        $exists = $stmt->fetchColumn() > 0;
        
        if ($exists) {
            // Update the push token
            $stmt = $conn->prepare("
                UPDATE pass_registrations 
                SET push_token = ?,
                    updated_at = NOW()
                WHERE serial_number = ? AND device_library_identifier = ? AND pass_type_identifier = ?
            ");
            $stmt->execute([$pushToken, $serialNumber, $deviceLibraryIdentifier, $passTypeIdentifier]);
            logMessage("Updated existing registration for device $deviceLibraryIdentifier");
        } else {
            // Create a new registration
            $stmt = $conn->prepare("
                INSERT INTO pass_registrations (
                    serial_number, 
                    device_library_identifier, 
                    pass_type_identifier,
                    pass_type_id,
                    push_token
                ) VALUES (?, ?, ?, ?, ?)
            ");
            $result = $stmt->execute([
                $serialNumber,
                $deviceLibraryIdentifier,
                $passTypeIdentifier,
                $passTypeIdentifier, // We're using the same value for both fields for now
                $pushToken
            ]);
            
            if (!$result) {
                logMessage("Error creating registration: " . print_r($stmt->errorInfo(), true));
                http_response_code(500);
                echo json_encode(['status' => 'error', 'message' => 'Database error during registration']);
                return;
            }
            
            logMessage("Created new registration for device $deviceLibraryIdentifier");
            
            // Make sure we have an entry in the pass_updates table
            $stmt = $conn->prepare("
                INSERT IGNORE INTO pass_updates (pass_id, last_updated) VALUES (?, NOW())
            ");
            $stmt->execute([$serialNumber]);
        }
        
        // Return success
        http_response_code(201); // Created
        echo json_encode(['status' => 'success']);
    } catch (Exception $e) {
        http_response_code(500);
        echo json_encode(['status' => 'error', 'message' => 'Database error: ' . $e->getMessage()]);
        logMessage('Database error during registration: ' . $e->getMessage());
    }
}

/**
 * Unregister a device directly via URL parameters
 */
function unregisterDeviceDirect($conn, $serialNumber, $deviceLibraryIdentifier) {
    logMessage("Unregistering device $deviceLibraryIdentifier from pass $serialNumber");
    
    try {
        // Delete the registration
        $stmt = $conn->prepare("
            DELETE FROM pass_registrations 
            WHERE serial_number = ? AND device_library_identifier = ?
        ");
        $stmt->execute([$serialNumber, $deviceLibraryIdentifier]);
        
        // Check if any row was affected
        if ($stmt->rowCount() > 0) {
            http_response_code(200);
            echo json_encode(['status' => 'success']);
            logMessage("Device $deviceLibraryIdentifier unregistered successfully");
        } else {
            http_response_code(404);
            echo json_encode(['status' => 'error', 'message' => 'Registration not found']);
            logMessage("Error: Registration not found for device $deviceLibraryIdentifier");
        }
    } catch (Exception $e) {
        http_response_code(500);
        echo json_encode(['status' => 'error', 'message' => 'Database error: ' . $e->getMessage()]);
        logMessage('Database error during unregistration: ' . $e->getMessage());
    }
}

/**
 * Get serial numbers for a device and pass type
 */
function getSerialNumbersForDevice($conn, $deviceLibraryIdentifier, $passTypeIdentifier) {
    logMessage("Getting serial numbers for device $deviceLibraryIdentifier and pass type $passTypeIdentifier");
    
    try {
        // Get the latest updated date from the If-Modified-Since header
        $ifModifiedSince = isset($_SERVER['HTTP_IF_MODIFIED_SINCE']) ? strtotime($_SERVER['HTTP_IF_MODIFIED_SINCE']) : null;
        
        // Get all passes registered for this device and pass type
        $query = "
            SELECT r.serial_number, u.last_updated
            FROM pass_registrations r
            LEFT JOIN pass_updates u ON r.serial_number = u.pass_id
            WHERE r.device_library_identifier = ?
            AND r.pass_type_identifier = ?
        ";
        
        $stmt = $conn->prepare($query);
        $stmt->execute([$deviceLibraryIdentifier, $passTypeIdentifier]);
        $passes = $stmt->fetchAll(PDO::FETCH_ASSOC);
        
        logMessage("Found " . count($passes) . " registered passes for device $deviceLibraryIdentifier");
        
        // Filter passes that have been updated since the If-Modified-Since date
        $serialNumbers = [];
        $mostRecentUpdate = 0;
        
        foreach ($passes as $pass) {
            $lastUpdated = isset($pass['last_updated']) ? strtotime($pass['last_updated']) : null;
            
            // Track the most recent update time
            if ($lastUpdated && $lastUpdated > $mostRecentUpdate) {
                $mostRecentUpdate = $lastUpdated;
            }
            
            // Include passes that don't have a last_updated date or were updated after If-Modified-Since
            if ($lastUpdated === null || $ifModifiedSince === null || $lastUpdated > $ifModifiedSince) {
                $serialNumbers[] = $pass['serial_number'];
            }
        }
        
        // If there are passes that need updates
        if (!empty($serialNumbers)) {
            // Use the most recent update time or current time if none exists
            $timestamp = $mostRecentUpdate > 0 ? $mostRecentUpdate : time();
            
            // Format timestamp in strict ISO8601/RFC3339 format with Z timezone indicator
            // This is the exact format Apple expects: YYYY-MM-DDTHH:MM:SSZ
            $lastUpdated = gmdate('Y-m-d\TH:i:s\Z', $timestamp);
            
            // Set the Last-Modified header to the current time in HTTP date format
            header('Last-Modified: ' . gmdate('D, d M Y H:i:s', $timestamp) . ' GMT');
            
            // Return the serial numbers and lastUpdated as JSON
            http_response_code(200);
            echo json_encode([
                'serialNumbers' => $serialNumbers,
                'lastUpdated' => $lastUpdated
            ]);
            logMessage("Returning " . count($serialNumbers) . " serial numbers for updates with lastUpdated: $lastUpdated");
        } else {
            // No passes need updates
            http_response_code(204); // No Content
            logMessage("No passes need updates");
        }
    } catch (Exception $e) {
        http_response_code(500);
        echo json_encode(['status' => 'error', 'message' => 'Database error: ' . $e->getMessage()]);
        logMessage('Database error getting serial numbers: ' . $e->getMessage());
    }
}

/**
 * Extract the authentication token from headers
 */
function extractAuthToken() {
    // First check traditional headers
    $headers = getallheaders();
    $authHeader = isset($headers['Authorization']) ? $headers['Authorization'] : '';
    
    if (strpos($authHeader, 'ApplePass ') === 0) {
        return substr($authHeader, 10); // Remove 'ApplePass ' prefix
    }
    
    // Then check HTTP_AUTHORIZATION (some servers convert headers to HTTP_* format)
    if (isset($_SERVER['HTTP_AUTHORIZATION']) && strpos($_SERVER['HTTP_AUTHORIZATION'], 'ApplePass ') === 0) {
        return substr($_SERVER['HTTP_AUTHORIZATION'], 10);
    }
    
    // Finally check if it was passed as a query parameter (not recommended but supported)
    if (isset($_GET['authorization'])) {
        return $_GET['authorization'];
    }
    
    return '';
} 