<?php
/**
 * PassKit Push Notification Sender
 * 
 * This file handles sending push notifications to registered devices using
 * the Apple Push Notification service (APNs) with JWT authentication.
 */

// Enable error reporting for debugging
ini_set('display_errors', 1);
ini_set('display_startup_errors', 1);
error_reporting(E_ALL);

// Set up logging
$logDir = __DIR__ . '/../../../logs';
if (!is_dir($logDir)) {
    mkdir($logDir, 0777, true);
}
$logFile = $logDir . '/passkit_push.log';

// Log function for debugging
function logMessage($message) {
    global $logFile;
    file_put_contents($logFile, date('Y-m-d H:i:s') . ' - ' . $message . "\n", FILE_APPEND);
}

// Include database functions
require_once __DIR__ . '/../../../includes/functions.php';
$conn = getDbConnection();

/**
 * Get JWT token for APNs authentication
 * 
 * @return string The JWT token
 */
function getApnsToken() {
    // Your key ID from Apple
    $keyId = 'Y8H9GBCK64';
    // Your team ID from Apple Developer account
    $teamId = 'C7CH6P3GLR';
    // Bundle ID of your pass
    $bundleId = 'pass.pointify.pk';
    
    // Path to your private key
    $keyPath = __DIR__ . '/../../../certs/AuthKey_Y8H9GBCK64.p8';
    
    // Read the key file
    $keyContents = file_get_contents($keyPath);
    if (!$keyContents) {
        logMessage("Error reading private key file at $keyPath");
        return false;
    }
    
    // Extract the private key from the PEM file
    $privateKey = openssl_pkey_get_private($keyContents);
    if (!$privateKey) {
        logMessage("Error loading private key: " . openssl_error_string());
        return false;
    }
    
    // JWT headers
    $headers = [
        'alg' => 'ES256',
        'kid' => $keyId
    ];
    
    // JWT payload
    $payload = [
        'iss' => $teamId,
        'iat' => time()
    ];
    
    // Encode the JWT headers and payload
    $encodedHeaders = base64UrlEncode(json_encode($headers));
    $encodedPayload = base64UrlEncode(json_encode($payload));
    
    // Create the signature input
    $signatureInput = $encodedHeaders . '.' . $encodedPayload;
    
    // Generate the signature
    $signature = '';
    openssl_sign($signatureInput, $signature, $privateKey, 'sha256');
    
    // Free the key resource
    openssl_pkey_free($privateKey);
    
    // Encode the signature
    $encodedSignature = base64UrlEncode($signature);
    
    // Create the JWT token
    $jwt = $signatureInput . '.' . $encodedSignature;
    
    return $jwt;
}

/**
 * Encode string for JWT
 * 
 * @param string $data The data to encode
 * @return string The encoded data
 */
function base64UrlEncode($data) {
    return rtrim(strtr(base64_encode($data), '+/', '-_'), '=');
}

/**
 * Push updates to all registered devices for a pass
 * 
 * @param string $passId The pass ID (card number)
 * @return array Status and message
 */
function pushPassUpdates($passId) {
    global $conn;
    logMessage("Pushing updates for pass: $passId");
    
    // Get all registered devices for this pass
    $stmt = $conn->prepare("
        SELECT device_library_identifier, push_token, pass_type_identifier 
        FROM pass_registrations 
        WHERE serial_number = ?
    ");
    $stmt->execute([$passId]);
    $devices = $stmt->fetchAll(PDO::FETCH_ASSOC);
    
    if (empty($devices)) {
        logMessage("No registered devices found for pass: $passId");
        return ['success' => false, 'message' => 'No registered devices found'];
    }
    
    logMessage("Found " . count($devices) . " registered device(s)");
    
    // Get the JWT token for APNs
    $jwt = getApnsToken();
    if (!$jwt) {
        logMessage("Failed to create JWT token");
        return ['success' => false, 'message' => 'Failed to create JWT token'];
    }
    
    // Log the JWT for debugging
    logMessage("Generated JWT token: " . substr($jwt, 0, 20) . "...");
    
    $successCount = 0;
    $failCount = 0;
    
    // Send push to each device
    foreach ($devices as $device) {
        $deviceLibraryIdentifier = $device['device_library_identifier'];
        $pushToken = $device['push_token'];
        $passTypeIdentifier = !empty($device['pass_type_identifier']) ? $device['pass_type_identifier'] : 'pass.pointify.pk';
        
        logMessage("Sending push to device: $deviceLibraryIdentifier with token: " . substr($pushToken, 0, 20) . "... for pass type: $passTypeIdentifier");
        
        // Production URL for APNs
        $url = "https://api.push.apple.com/3/device/$pushToken";
        
        // Use this for development/testing
        // $url = "https://api.development.push.apple.com/3/device/$pushToken";
        
        logMessage("Using APNS URL: $url");
        
        $ch = curl_init($url);
        curl_setopt($ch, CURLOPT_CUSTOMREQUEST, 'POST');
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false); // Set to false if you have SSL errors
        curl_setopt($ch, CURLOPT_HTTPHEADER, [
            'apns-topic: ' . $passTypeIdentifier, // Using the pass type identifier from the database
            'apns-push-type: background',
            'apns-priority: 10',
            'apns-expiration: 0',
            'authorization: bearer ' . $jwt
        ]);
        
        // Log all headers for debugging
        logMessage("Request headers: " . json_encode([
            'apns-topic: ' . $passTypeIdentifier,
            'apns-push-type: background',
            'apns-priority: 10',
            'apns-expiration: 0',
            'authorization: bearer ' . substr($jwt, 0, 20) . "..."
        ]));
        
        // Empty payload for passkit push
        curl_setopt($ch, CURLOPT_POSTFIELDS, '{}');
        
        $response = curl_exec($ch);
        $status = curl_getinfo($ch, CURLINFO_HTTP_CODE);
        $error = curl_error($ch);
        
        // Get detailed info about the request
        $info = curl_getinfo($ch);
        logMessage("CURL info: " . json_encode($info));
        
        logMessage("Push response status: $status, response: $response, error: $error");
        
        if ($status == 200) {
            $successCount++;
            // Update the last_pushed timestamp
            $stmt = $conn->prepare("
                UPDATE pass_updates 
                SET last_pushed = NOW() 
                WHERE pass_id = ?
            ");
            $stmt->execute([$passId]);
        } else {
            $failCount++;
            // Handle specific error cases
            if ($status == 410) { // Gone - token is no longer valid
                logMessage("Device token is no longer valid, removing registration: $deviceLibraryIdentifier");
                $stmt = $conn->prepare("
                    DELETE FROM pass_registrations 
                    WHERE device_library_identifier = ? AND serial_number = ?
                ");
                $stmt->execute([$deviceLibraryIdentifier, $passId]);
            }
        }
        
        curl_close($ch);
    }
    
    logMessage("Push completed for $passId: $successCount succeeded, $failCount failed");
    
    return [
        'success' => $successCount > 0,
        'message' => "Push notification sent to $successCount device(s), failed for $failCount device(s)"
    ];
}

// Handle API requests
if ($_SERVER['REQUEST_METHOD'] === 'POST') {
    $postData = json_decode(file_get_contents('php://input'), true);
    $passId = isset($postData['pass_id']) ? $postData['pass_id'] : '';
    
    if (empty($passId)) {
        logMessage("Missing pass_id in request");
        http_response_code(400);
        echo json_encode(['status' => 'error', 'message' => 'Pass ID is required']);
        exit;
    }
    
    logMessage("Processing push request for pass: $passId");
    $result = pushPassUpdates($passId);
    
    if ($result['success']) {
        echo json_encode(['status' => 'success', 'message' => $result['message']]);
    } else {
        http_response_code(500);
        echo json_encode(['status' => 'error', 'message' => $result['message']]);
    }
} else {
    logMessage("Method not allowed: " . $_SERVER['REQUEST_METHOD']);
    http_response_code(405);
    echo json_encode(['status' => 'error', 'message' => 'Method not allowed']);
} 