<?php
/**
 * Test Push Notification Script
 * 
 * This script allows you to test push notifications for a specific pass
 */

// Enable error reporting for debugging
ini_set('display_errors', 1);
ini_set('display_startup_errors', 1);
error_reporting(E_ALL);

// Set up logging
$logDir = __DIR__ . '/../../../logs';
if (!is_dir($logDir)) {
    mkdir($logDir, 0777, true);
}
$logFile = $logDir . '/test_push.log';

// Log function for debugging
function logMessage($message) {
    global $logFile;
    file_put_contents($logFile, date('Y-m-d H:i:s') . ' - ' . $message . "\n", FILE_APPEND);
}

// Include database functions
require_once __DIR__ . '/../../../includes/functions.php';

try {
    $conn = getDbConnection();
    $dbConnected = true;
} catch (Exception $e) {
    $dbConnected = false;
    logMessage("Database connection failed: " . $e->getMessage());
}

// Get card number from URL parameter
$cardNumber = isset($_GET['card']) ? $_GET['card'] : '';

// Generate HTML interface
header('Content-Type: text/html; charset=utf-8');
?>
<!DOCTYPE html>
<html>
<head>
    <title>Push Notification Test Tool</title>
    <style>
        body { font-family: Arial, sans-serif; margin: 20px; line-height: 1.6; }
        .container { max-width: 800px; margin: 0 auto; }
        .card { background: #f9f9f9; border-radius: 5px; padding: 20px; margin-bottom: 20px; box-shadow: 0 2px 4px rgba(0,0,0,0.1); }
        .success { color: green; }
        .error { color: red; }
        .info { color: blue; }
        input[type="text"] { padding: 8px; width: 200px; }
        button { padding: 8px 15px; background: #4CAF50; color: white; border: none; border-radius: 4px; cursor: pointer; }
        button:hover { background: #45a049; }
        ul { padding-left: 20px; }
    </style>
</head>
<body>
    <div class="container">
        <h1>Push Notification Test Tool</h1>
        
        <div class="card">
            <h2>Enter Card Number</h2>
            <form method="get">
                <input type="text" name="card" value="<?php echo htmlspecialchars($cardNumber); ?>" placeholder="Enter card number">
                <button type="submit">Check Card</button>
            </form>
        </div>
        
        <?php if (!empty($cardNumber)): ?>
        <div class="card">
            <h2>Test Results for Card: <?php echo htmlspecialchars($cardNumber); ?></h2>
            
            <div id="results">
                <!-- Test results will be displayed here -->

<?php
// Only continue with database operations if connection successful
if (!$dbConnected) {
    echo "<p class='error'>✗ Cannot proceed with tests because database connection failed.</p>";
    echo "</div></div></div></body></html>";
    exit;
}

// Test database connection
logMessage("Testing database connection");
echo "<h3>1. Database Connection</h3>";
try {
    if ($dbConnected && $conn) {
        echo "<p class='success'>✓ Connected to database successfully</p>";
        logMessage("Database connection successful");
    } else {
        echo "<p class='error'>✗ Failed to connect to database</p>";
        logMessage("Database connection failed");
    }
} catch (Exception $e) {
    echo "<p class='error'>✗ Database error: " . htmlspecialchars($e->getMessage()) . "</p>";
    logMessage("Database error: " . $e->getMessage());
}

// Check if card exists
logMessage("Checking if card exists: $cardNumber");
echo "<h3>2. Card Verification</h3>";
try {
    $stmt = $conn->prepare("SELECT id FROM loyalty_cards WHERE card_number = ?");
    $stmt->execute([$cardNumber]);
    $card = $stmt->fetch(PDO::FETCH_ASSOC);
    
    if ($card) {
        echo "<p class='success'>✓ Card found in database (ID: {$card['id']})</p>";
        logMessage("Card found: " . json_encode($card));
    } else {
        echo "<p class='error'>✗ Card not found in database</p>";
        logMessage("Card not found");
    }
} catch (Exception $e) {
    echo "<p class='error'>✗ Card check error: " . htmlspecialchars($e->getMessage()) . "</p>";
    logMessage("Card check error: " . $e->getMessage());
}

// Check for device registrations
logMessage("Checking device registrations for card: $cardNumber");
echo "<h3>3. Device Registrations</h3>";
try {
    $stmt = $conn->prepare("
        SELECT * FROM pass_registrations 
        WHERE serial_number = ?
    ");
    $stmt->execute([$cardNumber]);
    $registrations = $stmt->fetchAll(PDO::FETCH_ASSOC);
    
    if (count($registrations) > 0) {
        echo "<p class='success'>✓ Found " . count($registrations) . " device registration(s)</p>";
        echo "<ul>";
        foreach ($registrations as $reg) {
            echo "<li>Device: " . htmlspecialchars($reg['device_library_identifier']) . " (Push Token: " . htmlspecialchars(substr($reg['push_token'], 0, 10)) . "...)</li>";
        }
        echo "</ul>";
        logMessage("Found " . count($registrations) . " device registrations");
    } else {
        echo "<p class='error'>✗ No device registrations found</p>";
        logMessage("No device registrations");
    }
} catch (Exception $e) {
    echo "<p class='error'>✗ Registration check error: " . htmlspecialchars($e->getMessage()) . "</p>";
    logMessage("Registration check error: " . $e->getMessage());
}

// Check for pass updates
logMessage("Checking pass updates for card: $cardNumber");
echo "<h3>4. Pass Updates</h3>";
try {
    $stmt = $conn->prepare("
        SELECT * FROM pass_updates 
        WHERE pass_id = ?
    ");
    $stmt->execute([$cardNumber]);
    $updates = $stmt->fetch(PDO::FETCH_ASSOC);
    
    if ($updates) {
        echo "<p class='success'>✓ Pass updates record found</p>";
        echo "<ul>";
        echo "<li>Last change: " . htmlspecialchars($updates['last_updated']) . "</li>";
        echo "<li>Last push: " . htmlspecialchars($updates['last_pushed'] ?: 'Never') . "</li>";
        echo "</ul>";
        logMessage("Found pass updates: " . json_encode($updates));
    } else {
        echo "<p class='info'>ℹ No pass updates record found. Creating one...</p>";
        logMessage("No pass updates found, creating record");
        
        // Create a pass update record
        $stmt = $conn->prepare("
            INSERT INTO pass_updates (pass_id, last_updated)
            VALUES (?, NOW())
        ");
        if ($stmt->execute([$cardNumber])) {
            echo "<p class='success'>✓ Created pass updates record</p>";
            logMessage("Created pass updates record");
        } else {
            echo "<p class='error'>✗ Failed to create pass updates record</p>";
            logMessage("Failed to create pass updates record");
        }
    }
} catch (Exception $e) {
    echo "<p class='error'>✗ Pass updates check error: " . htmlspecialchars($e->getMessage()) . "</p>";
    logMessage("Pass updates check error: " . $e->getMessage());
}

// Send push notification (with form)
echo "<h3>5. Send Push Notification</h3>";

if (isset($_POST['send_push']) && $_POST['card_number'] == $cardNumber) {
    logMessage("Manually sending push for card: $cardNumber");
    
    // Get the push.php file URL
    $pushUrl = 'http' . (isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] === 'on' ? 's' : '') . '://' . 
               $_SERVER['HTTP_HOST'] . 
               dirname($_SERVER['PHP_SELF']) . '/push.php';
    
    // Setup cURL request
    $ch = curl_init($pushUrl);
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
    curl_setopt($ch, CURLOPT_POST, true);
    curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode(['pass_id' => $cardNumber]));
    curl_setopt($ch, CURLOPT_HTTPHEADER, ['Content-Type: application/json']);
    
    // Execute request
    $response = curl_exec($ch);
    $status = curl_getinfo($ch, CURLINFO_HTTP_CODE);
    $error = curl_error($ch);
    curl_close($ch);
    
    // Output results
    logMessage("Push response ($status): $response, Error: $error");
    
    if ($status == 200) {
        $result = json_decode($response, true);
        if ($result && isset($result['status']) && $result['status'] == 'success') {
            echo "<p class='success'>✓ Push notification sent: " . htmlspecialchars($result['message']) . "</p>";
        } else {
            echo "<p class='error'>✗ Push failed: " . htmlspecialchars($response) . "</p>";
        }
    } else {
        echo "<p class='error'>✗ Push request failed (HTTP $status): " . htmlspecialchars($error ?: $response) . "</p>";
    }
}

// Show push form
echo '<form method="post">';
echo '<input type="hidden" name="card_number" value="' . htmlspecialchars($cardNumber) . '">';
echo '<button type="submit" name="send_push">Send Push Notification</button>';
echo '</form>';

// Update stamp count (with form)
echo "<h3>6. Update Stamp Count</h3>";

if (isset($_POST['update_stamps']) && $_POST['card_number'] == $cardNumber) {
    $newStamps = isset($_POST['stamp_count']) ? (int)$_POST['stamp_count'] : 0;
    logMessage("Updating stamp count for card $cardNumber to $newStamps");
    
    try {
        // First update the card's stamp count
        $stmt = $conn->prepare("
            UPDATE loyalty_cards 
            SET stamp_count = ? 
            WHERE card_number = ?
        ");
        $success = $stmt->execute([$newStamps, $cardNumber]);
        
        // Then update the pass_updates timestamp
        if ($success) {
            $stmt = $conn->prepare("
                UPDATE pass_updates 
                SET last_updated = NOW() 
                WHERE pass_id = ?
            ");
            $stmt->execute([$cardNumber]);
            
            echo "<p class='success'>✓ Updated stamp count to $newStamps and triggered pass update</p>";
            logMessage("Successfully updated stamp count to $newStamps");
        } else {
            echo "<p class='error'>✗ Failed to update stamp count</p>";
            logMessage("Failed to update stamp count");
        }
    } catch (Exception $e) {
        echo "<p class='error'>✗ Error updating stamp count: " . htmlspecialchars($e->getMessage()) . "</p>";
        logMessage("Error updating stamp count: " . $e->getMessage());
    }
}

// Show stamps form
try {
    $stmt = $conn->prepare("SELECT stamp_count FROM loyalty_cards WHERE card_number = ?");
    $stmt->execute([$cardNumber]);
    $cardData = $stmt->fetch(PDO::FETCH_ASSOC);
    $currentStamps = $cardData ? $cardData['stamp_count'] : 0;
    
    echo '<form method="post">';
    echo '<input type="hidden" name="card_number" value="' . htmlspecialchars($cardNumber) . '">';
    echo '<label>New Stamp Count: <input type="number" name="stamp_count" value="' . $currentStamps . '" min="0" max="10"></label> ';
    echo '<button type="submit" name="update_stamps">Update Stamps</button>';
    echo '</form>';
} catch (Exception $e) {
    echo "<p class='error'>✗ Error getting current stamps: " . htmlspecialchars($e->getMessage()) . "</p>";
    logMessage("Error getting current stamps: " . $e->getMessage());
}
?>

            </div>
        </div>
        <?php endif; ?>
    </div>
</body>
</html>
<?php
// Catch any uncaught exceptions
try {
    // Page execution completes normally
} catch (Exception $e) {
    logMessage("Uncaught exception: " . $e->getMessage());
    echo "<div class='error'>Fatal error: " . htmlspecialchars($e->getMessage()) . "</div>";
}
?> 