<?php
// Enable error reporting for debugging
ini_set('display_errors', 1);
ini_set('display_startup_errors', 1);
error_reporting(E_ALL);

// Set up logging
$logDir = __DIR__ . '/../../../logs';
if (!is_dir($logDir)) {
    mkdir($logDir, 0777, true);
}
$logFile = $logDir . '/test_push_single.log';

// Log function for debugging
function logMessage($message) {
    global $logFile;
    file_put_contents($logFile, date('Y-m-d H:i:s') . ' - ' . $message . "\n", FILE_APPEND);
}

// Include database functions
require_once __DIR__ . '/../../../includes/functions.php';
$conn = getDbConnection();

// Get card number and device ID from URL parameters
$cardNumber = isset($_GET['card']) ? $_GET['card'] : 'LC202503233546';
$deviceId = isset($_GET['device']) ? $_GET['device'] : '';

// Find a device if not specified
if (empty($deviceId)) {
    try {
        $stmt = $conn->prepare("
            SELECT device_library_identifier 
            FROM pass_registrations 
            WHERE serial_number = ? 
            ORDER BY updated_at DESC 
            LIMIT 1
        ");
        $stmt->execute([$cardNumber]);
        $deviceId = $stmt->fetchColumn();
        
        if (!$deviceId) {
            die("No registered devices found for card: $cardNumber");
        }
    } catch (Exception $e) {
        die("Database error: " . $e->getMessage());
    }
}

// Get pass type identifier from database
try {
    $stmt = $conn->prepare("
        SELECT pass_type_identifier 
        FROM pass_registrations 
        WHERE device_library_identifier = ? AND serial_number = ?
        LIMIT 1
    ");
    $stmt->execute([$deviceId, $cardNumber]);
    $passTypeId = $stmt->fetchColumn() ?: 'pass.pointify.pk';
} catch (Exception $e) {
    $passTypeId = 'pass.pointify.pk';
    logMessage("Error getting pass type: " . $e->getMessage());
}

// Get push token from database
try {
    $stmt = $conn->prepare("
        SELECT push_token 
        FROM pass_registrations 
        WHERE device_library_identifier = ? AND serial_number = ?
        LIMIT 1
    ");
    $stmt->execute([$deviceId, $cardNumber]);
    $pushToken = $stmt->fetchColumn();
    
    if (!$pushToken) {
        die("No push token found for device: $deviceId and card: $cardNumber");
    }
} catch (Exception $e) {
    die("Database error: " . $e->getMessage());
}

// Update the pass in the database
try {
    // Update the version and timestamp in pass_updates
    $stmt = $conn->prepare("
        UPDATE pass_updates 
        SET last_updated = NOW(), 
            version = version + 1
        WHERE pass_id = ?
    ");
    $stmt->execute([$cardNumber]);
    
    if ($stmt->rowCount() === 0) {
        // Insert if it doesn't exist
        $stmt = $conn->prepare("
            INSERT INTO pass_updates (pass_id, last_updated, version)
            VALUES (?, NOW(), 1)
        ");
        $stmt->execute([$cardNumber]);
    }
    
    logMessage("Updated pass version for card: $cardNumber");
} catch (Exception $e) {
    logMessage("Error updating pass version: " . $e->getMessage());
}

// Generate JWT token for APNs
require_once __DIR__ . '/push.php';
$jwt = getApnsToken();

if (!$jwt) {
    die("Failed to create JWT token for APNs");
}

// Send push notification
$url = "https://api.push.apple.com/3/device/" . $pushToken;

// Set up request headers
$headers = [
    "apns-topic: " . $passTypeId,
    "apns-push-type: background",
    "apns-priority: 10",
    "apns-expiration: 0",
    "authorization: bearer " . $jwt
];

// Set up cURL request
$ch = curl_init($url);
curl_setopt($ch, CURLOPT_HTTP_VERSION, CURL_HTTP_VERSION_2_0);
curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
curl_setopt($ch, CURLOPT_POST, true);
curl_setopt($ch, CURLOPT_POSTFIELDS, '{}'); // Empty payload is fine for passkit
curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false); // For development only

// Execute request
$response = curl_exec($ch);
$status = curl_getinfo($ch, CURLINFO_HTTP_CODE);
$info = curl_getinfo($ch);
$error = curl_error($ch);
curl_close($ch);

// Update last_pushed timestamp
if ($status == 200) {
    try {
        $stmt = $conn->prepare("
            UPDATE pass_updates 
            SET last_pushed = NOW() 
            WHERE pass_id = ?
        ");
        $stmt->execute([$cardNumber]);
        logMessage("Updated last_pushed timestamp for card: $cardNumber");
    } catch (Exception $e) {
        logMessage("Error updating last_pushed timestamp: " . $e->getMessage());
    }
}

// Log push results
logMessage("Push notification sent with status: $status");
logMessage("Response: " . $response);
if ($error) {
    logMessage("cURL error: " . $error);
}
logMessage("cURL info: " . json_encode($info));

// Output human-readable result
header('Content-Type: text/html; charset=utf-8');
?>
<!DOCTYPE html>
<html>
<head>
    <title>Push Notification Test</title>
    <style>
        body { font-family: Arial, sans-serif; margin: 20px; line-height: 1.6; }
        .container { max-width: 800px; margin: 0 auto; padding: 20px; background: #f9f9f9; border-radius: 5px; }
        .success { color: green; }
        .error { color: red; }
        .info { color: blue; }
        pre { background: #f5f5f5; padding: 10px; border-radius: 5px; overflow-x: auto; }
    </style>
</head>
<body>
    <div class="container">
        <h1>Push Notification Test</h1>
        
        <h2>Push Details</h2>
        <ul>
            <li><strong>Card Number:</strong> <?php echo htmlspecialchars($cardNumber); ?></li>
            <li><strong>Device ID:</strong> <?php echo htmlspecialchars(substr($deviceId, 0, 10) . '...'); ?></li>
            <li><strong>Pass Type:</strong> <?php echo htmlspecialchars($passTypeId); ?></li>
            <li><strong>Push Token:</strong> <?php echo htmlspecialchars(substr($pushToken, 0, 10) . '...'); ?></li>
        </ul>
        
        <h2>Push Result</h2>
        <?php if ($status == 200): ?>
            <p class="success">Push notification sent successfully!</p>
        <?php else: ?>
            <p class="error">Failed to send push notification. Status code: <?php echo $status; ?></p>
        <?php endif; ?>
        
        <?php if ($response): ?>
            <h3>Response:</h3>
            <pre><?php echo htmlspecialchars($response); ?></pre>
        <?php endif; ?>
        
        <?php if ($error): ?>
            <h3>Error:</h3>
            <pre><?php echo htmlspecialchars($error); ?></pre>
        <?php endif; ?>
        
        <h2>Next Steps</h2>
        <p>If the push was successful:</p>
        <ol>
            <li>Your Apple Wallet should receive the notification within a few seconds</li>
            <li>The wallet will then query for updated passes using the getSerialNumbers endpoint</li>
            <li>It will then download the updated pass and refresh the UI</li>
        </ol>
        
        <p>If you don't see the update, you can:</p>
        <ol>
            <li><a href="test_push.php?card=<?php echo urlencode($cardNumber); ?>">Try using the full test tool</a></li>
            <li><a href="debug.php">Check the debug page</a> to verify registration</li>
            <li>Verify Apple Push certificates are valid</li>
        </ol>
    </div>
</body>
</html> 