<?php
// Enable error reporting for debugging
ini_set('display_errors', 1);
ini_set('display_startup_errors', 1);
error_reporting(E_ALL);

// Set up logging
$logDir = __DIR__ . '/../../../logs';
if (!is_dir($logDir)) {
    mkdir($logDir, 0777, true);
}
$logFile = $logDir . '/update_card.log';

// Log function for debugging
function logMessage($message) {
    global $logFile;
    file_put_contents($logFile, date('Y-m-d H:i:s') . ' - ' . $message . "\n", FILE_APPEND);
}

// Include database functions
require_once __DIR__ . '/../../../includes/functions.php';
$conn = getDbConnection();

// Get card number from URL parameter
$cardNumber = isset($_GET['card']) ? $_GET['card'] : 'LC202503233546';
$newStampCount = isset($_GET['stamps']) ? (int)$_GET['stamps'] : 0;
$increaseBy = isset($_GET['increase']) ? (int)$_GET['increase'] : 0;

// Get current stamp count if not specified
if ($newStampCount === 0 && $increaseBy > 0) {
    try {
        $stmt = $conn->prepare("SELECT stamp_count FROM loyalty_cards WHERE card_number = ?");
        $stmt->execute([$cardNumber]);
        $currentStamps = (int)$stmt->fetchColumn();
        $newStampCount = $currentStamps + $increaseBy;
    } catch (Exception $e) {
        logMessage("Error getting current stamp count: " . $e->getMessage());
        $newStampCount = $increaseBy; // Fallback
    }
}

// Update the card in the database
try {
    $success = false;
    
    // Update the loyalty card
    $stmt = $conn->prepare("
        UPDATE loyalty_cards 
        SET stamp_count = ? 
        WHERE card_number = ?
    ");
    $success = $stmt->execute([$newStampCount, $cardNumber]);
    
    if (!$success) {
        throw new Exception("Failed to update loyalty card");
    }
    
    if ($stmt->rowCount() === 0) {
        throw new Exception("Card not found: $cardNumber");
    }
    
    logMessage("Updated stamp count to $newStampCount for card: $cardNumber");
    
    // Update the pass_updates table
    $stmt = $conn->prepare("
        UPDATE pass_updates 
        SET last_updated = NOW(),
            version = version + 1
        WHERE pass_id = ?
    ");
    $stmt->execute([$cardNumber]);
    
    if ($stmt->rowCount() === 0) {
        // Insert if it doesn't exist
        $stmt = $conn->prepare("
            INSERT INTO pass_updates (pass_id, last_updated, version)
            VALUES (?, NOW(), 1)
        ");
        $stmt->execute([$cardNumber]);
    }
    
    // Force regenerate the pass file
    $passFilePath = __DIR__ . '/../../../passes/' . $cardNumber . '.pkpass';
    if (file_exists($passFilePath)) {
        unlink($passFilePath);
        logMessage("Deleted existing pass file to force regeneration");
    }
    
    $updateSuccessful = true;
    logMessage("Successfully updated card and triggered pass update");
} catch (Exception $e) {
    $updateSuccessful = false;
    logMessage("Error updating card: " . $e->getMessage());
}

// Now send push notifications to all registered devices
try {
    // Get all registered devices for this card
    $stmt = $conn->prepare("
        SELECT device_library_identifier, push_token, pass_type_identifier
        FROM pass_registrations
        WHERE serial_number = ?
    ");
    $stmt->execute([$cardNumber]);
    $devices = $stmt->fetchAll(PDO::FETCH_ASSOC);
    
    if (empty($devices)) {
        logMessage("No registered devices found for card: $cardNumber");
        $pushSuccessful = false;
    } else {
        logMessage("Found " . count($devices) . " registered device(s)");
        
        // Generate JWT token for APNs
        require_once __DIR__ . '/push.php';
        $jwt = getApnsToken();
        
        if (!$jwt) {
            throw new Exception("Failed to create JWT token for APNs");
        }
        
        $successCount = 0;
        $failCount = 0;
        
        foreach ($devices as $device) {
            $deviceId = $device['device_library_identifier'];
            $pushToken = $device['push_token'];
            $passTypeId = $device['pass_type_identifier'];
            
            logMessage("Sending push to device: " . substr($deviceId, 0, 10) . "... with token: " . substr($pushToken, 0, 10) . "...");
            
            // Send push notification
            $url = "https://api.push.apple.com/3/device/" . $pushToken;
            
            // Set up request headers
            $headers = [
                "apns-topic: " . $passTypeId,
                "apns-push-type: background",
                "apns-priority: 10",
                "apns-expiration: 0",
                "authorization: bearer " . $jwt
            ];
            
            // Set up cURL request
            $ch = curl_init($url);
            curl_setopt($ch, CURLOPT_HTTP_VERSION, CURL_HTTP_VERSION_2_0);
            curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
            curl_setopt($ch, CURLOPT_POST, true);
            curl_setopt($ch, CURLOPT_POSTFIELDS, '{}'); // Empty payload is fine for passkit
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
            curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false); // For development only
            
            // Execute request
            $response = curl_exec($ch);
            $status = curl_getinfo($ch, CURLINFO_HTTP_CODE);
            $error = curl_error($ch);
            curl_close($ch);
            
            if ($status == 200) {
                $successCount++;
                logMessage("Push succeeded for device: " . substr($deviceId, 0, 10) . "...");
            } else {
                $failCount++;
                logMessage("Push failed for device: " . substr($deviceId, 0, 10) . "... Status: $status, Error: $error");
            }
        }
        
        // Update last_pushed timestamp
        if ($successCount > 0) {
            $stmt = $conn->prepare("
                UPDATE pass_updates 
                SET last_pushed = NOW() 
                WHERE pass_id = ?
            ");
            $stmt->execute([$cardNumber]);
            logMessage("Updated last_pushed timestamp for card: $cardNumber");
        }
        
        $pushSuccessful = $successCount > 0;
        logMessage("Push notification results: $successCount succeeded, $failCount failed");
    }
} catch (Exception $e) {
    $pushSuccessful = false;
    logMessage("Error sending push notifications: " . $e->getMessage());
}

// Output human-readable result
header('Content-Type: text/html; charset=utf-8');
?>
<!DOCTYPE html>
<html>
<head>
    <title>Update Card and Push</title>
    <style>
        body { font-family: Arial, sans-serif; margin: 20px; line-height: 1.6; }
        .container { max-width: 800px; margin: 0 auto; padding: 20px; background: #f9f9f9; border-radius: 5px; }
        .success { color: green; }
        .error { color: red; }
        .info { color: blue; }
        .form-group { margin-bottom: 15px; }
        input, button { padding: 8px; font-size: 16px; }
        button { background: #4CAF50; color: white; border: none; cursor: pointer; border-radius: 4px; }
        button:hover { background: #45a049; }
    </style>
</head>
<body>
    <div class="container">
        <h1>Update Card and Push</h1>
        
        <div class="form-group">
            <form method="get">
                <label for="card">Card Number:</label>
                <input type="text" id="card" name="card" value="<?php echo htmlspecialchars($cardNumber); ?>" required>
                <br><br>
                <label for="stamps">New Stamp Count:</label>
                <input type="number" id="stamps" name="stamps" value="<?php echo $newStampCount; ?>" min="0" max="10">
                <br><br>
                <span>OR</span>
                <br><br>
                <label for="increase">Increase Stamps By:</label>
                <input type="number" id="increase" name="increase" value="1" min="1" max="10">
                <br><br>
                <button type="submit">Update Card & Send Push</button>
            </form>
        </div>
        
        <div class="results">
            <h2>Results</h2>
            
            <?php if ($updateSuccessful): ?>
                <p class="success">✓ Card successfully updated with <?php echo $newStampCount; ?> stamps</p>
            <?php else: ?>
                <p class="error">✗ Failed to update card</p>
            <?php endif; ?>
            
            <?php if (isset($pushSuccessful)): ?>
                <?php if ($pushSuccessful): ?>
                    <p class="success">✓ Push notifications sent successfully</p>
                    <?php if (isset($successCount) && isset($failCount)): ?>
                        <p>Sent to <?php echo $successCount; ?> device(s), failed for <?php echo $failCount; ?> device(s)</p>
                    <?php endif; ?>
                <?php else: ?>
                    <p class="error">✗ Failed to send push notifications</p>
                <?php endif; ?>
            <?php endif; ?>
        </div>
        
        <div class="next-steps">
            <h2>Next Steps</h2>
            
            <p>Check your Apple Wallet - the pass should update automatically within a few seconds.</p>
            
            <p>If the pass doesn't update:</p>
            <ol>
                <li>Make sure the device is registered (<a href="debug.php">Check debug page</a>)</li>
                <li>Verify your APNs certificates are valid and not expired</li>
                <li>Check that your device is online and has notifications enabled</li>
                <li>Try <a href="test_push_single.php?card=<?php echo urlencode($cardNumber); ?>">sending a push directly</a></li>
            </ol>
            
            <p><a href="test_push.php?card=<?php echo urlencode($cardNumber); ?>">Go to full test page</a></p>
        </div>
    </div>
</body>
</html> 